# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/automation/v1/action.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from _qwak_proto.qwak.automation.v1 import common_pb2 as qwak_dot_automation_dot_v1_dot_common__pb2
from _qwak_proto.qwak.automation.v1 import auto_scaling_pb2 as qwak_dot_automation_dot_v1_dot_auto__scaling__pb2
from _qwak_proto.qwak.batch_job.v1 import batch_job_service_pb2 as qwak_dot_batch__job_dot_v1_dot_batch__job__service__pb2
from _qwak_proto.qwak.user_application.common.v0 import resources_pb2 as qwak_dot_user__application_dot_common_dot_v0_dot_resources__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fqwak/automation/v1/action.proto\x12\x12qwak.automation.v1\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1fqwak/automation/v1/common.proto\x1a%qwak/automation/v1/auto_scaling.proto\x1a)qwak/batch_job/v1/batch_job_service.proto\x1a/qwak/user_application/common/v0/resources.proto\"\x99\x01\n\x06\x41\x63tion\x12@\n\x0c\x62uild_deploy\x18\x01 \x01(\x0b\x32(.qwak.automation.v1.BuildAndDeployActionH\x00\x12\x43\n\x0f\x62\x61tch_execution\x18\x02 \x01(\x0b\x32(.qwak.automation.v1.BatchExecutionActionH\x00\x42\x08\n\x06\x61\x63tion\"Q\n\x14\x42\x61tchExecutionAction\x12\x39\n\x11\x65xecute_batch_job\x18\x01 \x01(\x0b\x32\x1e.qwak.batchjob.BatchJobRequest\"\xcd\x01\n\x14\x42uildAndDeployAction\x12\x31\n\nbuild_spec\x18\x01 \x01(\x0b\x32\x1d.qwak.automation.v1.BuildSpec\x12\x45\n\x14\x64\x65ployment_condition\x18\x02 \x01(\x0b\x32\'.qwak.automation.v1.DeploymentCondition\x12;\n\x0f\x64\x65ployment_spec\x18\x03 \x01(\x0b\x32\".qwak.automation.v1.DeploymentSpec\"d\n\x13\x44\x65ploymentCondition\x12@\n\x0c\x62uild_metric\x18\x01 \x01(\x0b\x32(.qwak.automation.v1.BuildMetricConditionH\x00\x42\x0b\n\tcondition\"\x89\x01\n\x14\x42uildMetricCondition\x12\x13\n\x0bmetric_name\x18\x01 \x01(\t\x12I\n\x13threshold_direction\x18\x02 \x01(\x0e\x32,.qwak.automation.v1.MetricThresholdDirection\x12\x11\n\tthreshold\x18\x03 \x01(\t\"\xdf\x01\n\x0e\x44\x65ploymentSpec\x12;\n\x0f\x64\x65ployment_size\x18\x01 \x01(\x0b\x32\".qwak.automation.v1.DeploymentSize\x12G\n\x10\x61\x64vanced_options\x18\x02 \x01(\x0b\x32-.qwak.automation.v1.AdvancedDeploymentOptions\x12\x1f\n\x17selected_variation_name\x18\x03 \x01(\t\x12\x14\n\x0c\x65nvironments\x18\x04 \x03(\t\x12\x10\n\x08\x65nv_vars\x18\x05 \x03(\t\"\xc4\x02\n\x19\x41\x64vancedDeploymentOptions\x12%\n\x1dnumber_of_http_server_workers\x18\x01 \x01(\x05\x12\x1f\n\x17http_request_timeout_ms\x18\x02 \x01(\x05\x12\x13\n\x0b\x64\x61\x65mon_mode\x18\x03 \x01(\x08\x12\x1b\n\x13\x63ustom_iam_role_arn\x18\x04 \x01(\t\x12\x42\n\x13\x61uto_scaling_config\x18\x05 \x01(\x0b\x32%.qwak.automation.v1.AutoScalingConfig\x12\x16\n\x0emax_batch_size\x18\x06 \x01(\x05\x12(\n deployment_process_timeout_limit\x18\x07 \x01(\x05\x12\'\n\x1fservice_account_key_secret_name\x18\x08 \x01(\t\"\xad\x02\n\x0e\x44\x65ploymentSize\x12\x16\n\x0enumber_of_pods\x18\x01 \x01(\x05\x12\x0f\n\x03\x63pu\x18\x02 \x01(\x02\x42\x02\x18\x01\x12\x19\n\rmemory_amount\x18\x03 \x01(\x05\x42\x02\x18\x01\x12\x38\n\x0cmemory_units\x18\x04 \x01(\x0e\x32\x1e.qwak.automation.v1.MemoryUnitB\x02\x18\x01\x12;\n\rgpu_resources\x18\x05 \x01(\x0b\x32 .qwak.automation.v1.GpuResourcesB\x02\x18\x01\x12`\n\x1c\x63lient_pod_compute_resources\x18\x06 \x01(\x0b\x32:.qwak.user_application.common.v0.ClientPodComputeResources\"\xc0\x04\n\tBuildSpec\x12<\n\x10git_model_source\x18\x01 \x01(\x0b\x32\".qwak.automation.v1.GitModelSource\x12\x41\n\nparameters\x18\x02 \x03(\x0b\x32-.qwak.automation.v1.BuildSpec.ParametersEntry\x12\x0c\n\x04tags\x18\x03 \x03(\t\x12\x12\n\nbase_image\x18\x04 \x01(\t\x12\x18\n\x10\x61ssumed_iam_role\x18\x05 \x01(\t\x12/\n\x08resource\x18\x06 \x01(\x0b\x32\x1d.qwak.automation.v1.Resources\x12\x10\n\x08main_dir\x18\x07 \x01(\t\x12\x1c\n\x14\x64\x65pendency_file_path\x18\x08 \x01(\t\x12\x10\n\x08\x65nv_vars\x18\t \x03(\t\x12\x16\n\x0egpu_compatible\x18\n \x01(\x08\x12.\n\npush_image\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12;\n\x0fpurchase_option\x18\x0c \x01(\x0e\x32\".qwak.automation.v1.PurchaseOption\x12\'\n\x1fservice_account_key_secret_name\x18\r \x01(\t\x12\"\n\x1aprovision_instance_timeout\x18\x0e \x01(\x05\x1a\x31\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xf9\x01\n\tResources\x12=\n\rcpu_resources\x18\x01 \x01(\x0b\x32 .qwak.automation.v1.CpuResourcesB\x02\x18\x01H\x00\x12=\n\rgpu_resources\x18\x02 \x01(\x0b\x32 .qwak.automation.v1.GpuResourcesB\x02\x18\x01H\x00\x12\x62\n\x1c\x63lient_pod_compute_resources\x18\x03 \x01(\x0b\x32:.qwak.user_application.common.v0.ClientPodComputeResourcesH\x00\x42\n\n\x08resource\"h\n\x0c\x43puResources\x12\x0b\n\x03\x63pu\x18\x01 \x01(\x02\x12\x15\n\rmemory_amount\x18\x02 \x01(\x05\x12\x34\n\x0cmemory_units\x18\x03 \x01(\x0e\x32\x1e.qwak.automation.v1.MemoryUnit\"Q\n\x0cGpuResources\x12-\n\x08gpu_type\x18\x01 \x01(\x0e\x32\x1b.qwak.automation.v1.GpuType\x12\x12\n\ngpu_amount\x18\x02 \x01(\x05\"w\n\x0eGitModelSource\x12\x0f\n\x07git_uri\x18\x01 \x01(\t\x12#\n\x1bgit_credentials_secret_name\x18\x02 \x01(\t\x12\x12\n\ngit_branch\x18\x03 \x01(\t\x12\x1b\n\x13git_ssh_secret_name\x18\x04 \x01(\t*w\n\x07GpuType\x12\x0b\n\x07INVALID\x10\x00\x12\x0e\n\nNVIDIA_K80\x10\x01\x12\x0f\n\x0bNVIDIA_V100\x10\x02\x12\x0f\n\x0bNVIDIA_A100\x10\x03\x12\r\n\tNVIDIA_T4\x10\x04\x12\x0f\n\x0bNVIDIA_A10G\x10\x05\x12\r\n\tNVIDIA_L4\x10\x06*+\n\nMemoryUnit\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03MIB\x10\x01\x12\x07\n\x03GIB\x10\x02*)\n\x0ePurchaseOption\x12\r\n\tON_DEMAND\x10\x00\x12\x08\n\x04SPOT\x10\x01\x42!\n\x1d\x63om.qwak.ai.automation.api.v1P\x01\x62\x06proto3')

_GPUTYPE = DESCRIPTOR.enum_types_by_name['GpuType']
GpuType = enum_type_wrapper.EnumTypeWrapper(_GPUTYPE)
_MEMORYUNIT = DESCRIPTOR.enum_types_by_name['MemoryUnit']
MemoryUnit = enum_type_wrapper.EnumTypeWrapper(_MEMORYUNIT)
_PURCHASEOPTION = DESCRIPTOR.enum_types_by_name['PurchaseOption']
PurchaseOption = enum_type_wrapper.EnumTypeWrapper(_PURCHASEOPTION)
INVALID = 0
NVIDIA_K80 = 1
NVIDIA_V100 = 2
NVIDIA_A100 = 3
NVIDIA_T4 = 4
NVIDIA_A10G = 5
NVIDIA_L4 = 6
UNKNOWN = 0
MIB = 1
GIB = 2
ON_DEMAND = 0
SPOT = 1


_ACTION = DESCRIPTOR.message_types_by_name['Action']
_BATCHEXECUTIONACTION = DESCRIPTOR.message_types_by_name['BatchExecutionAction']
_BUILDANDDEPLOYACTION = DESCRIPTOR.message_types_by_name['BuildAndDeployAction']
_DEPLOYMENTCONDITION = DESCRIPTOR.message_types_by_name['DeploymentCondition']
_BUILDMETRICCONDITION = DESCRIPTOR.message_types_by_name['BuildMetricCondition']
_DEPLOYMENTSPEC = DESCRIPTOR.message_types_by_name['DeploymentSpec']
_ADVANCEDDEPLOYMENTOPTIONS = DESCRIPTOR.message_types_by_name['AdvancedDeploymentOptions']
_DEPLOYMENTSIZE = DESCRIPTOR.message_types_by_name['DeploymentSize']
_BUILDSPEC = DESCRIPTOR.message_types_by_name['BuildSpec']
_BUILDSPEC_PARAMETERSENTRY = _BUILDSPEC.nested_types_by_name['ParametersEntry']
_RESOURCES = DESCRIPTOR.message_types_by_name['Resources']
_CPURESOURCES = DESCRIPTOR.message_types_by_name['CpuResources']
_GPURESOURCES = DESCRIPTOR.message_types_by_name['GpuResources']
_GITMODELSOURCE = DESCRIPTOR.message_types_by_name['GitModelSource']
Action = _reflection.GeneratedProtocolMessageType('Action', (_message.Message,), {
  'DESCRIPTOR' : _ACTION,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.Action)
  })
_sym_db.RegisterMessage(Action)

BatchExecutionAction = _reflection.GeneratedProtocolMessageType('BatchExecutionAction', (_message.Message,), {
  'DESCRIPTOR' : _BATCHEXECUTIONACTION,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.BatchExecutionAction)
  })
_sym_db.RegisterMessage(BatchExecutionAction)

BuildAndDeployAction = _reflection.GeneratedProtocolMessageType('BuildAndDeployAction', (_message.Message,), {
  'DESCRIPTOR' : _BUILDANDDEPLOYACTION,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.BuildAndDeployAction)
  })
_sym_db.RegisterMessage(BuildAndDeployAction)

DeploymentCondition = _reflection.GeneratedProtocolMessageType('DeploymentCondition', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTCONDITION,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.DeploymentCondition)
  })
_sym_db.RegisterMessage(DeploymentCondition)

BuildMetricCondition = _reflection.GeneratedProtocolMessageType('BuildMetricCondition', (_message.Message,), {
  'DESCRIPTOR' : _BUILDMETRICCONDITION,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.BuildMetricCondition)
  })
_sym_db.RegisterMessage(BuildMetricCondition)

DeploymentSpec = _reflection.GeneratedProtocolMessageType('DeploymentSpec', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTSPEC,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.DeploymentSpec)
  })
_sym_db.RegisterMessage(DeploymentSpec)

AdvancedDeploymentOptions = _reflection.GeneratedProtocolMessageType('AdvancedDeploymentOptions', (_message.Message,), {
  'DESCRIPTOR' : _ADVANCEDDEPLOYMENTOPTIONS,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.AdvancedDeploymentOptions)
  })
_sym_db.RegisterMessage(AdvancedDeploymentOptions)

DeploymentSize = _reflection.GeneratedProtocolMessageType('DeploymentSize', (_message.Message,), {
  'DESCRIPTOR' : _DEPLOYMENTSIZE,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.DeploymentSize)
  })
_sym_db.RegisterMessage(DeploymentSize)

BuildSpec = _reflection.GeneratedProtocolMessageType('BuildSpec', (_message.Message,), {

  'ParametersEntry' : _reflection.GeneratedProtocolMessageType('ParametersEntry', (_message.Message,), {
    'DESCRIPTOR' : _BUILDSPEC_PARAMETERSENTRY,
    '__module__' : 'qwak.automation.v1.action_pb2'
    # @@protoc_insertion_point(class_scope:qwak.automation.v1.BuildSpec.ParametersEntry)
    })
  ,
  'DESCRIPTOR' : _BUILDSPEC,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.BuildSpec)
  })
_sym_db.RegisterMessage(BuildSpec)
_sym_db.RegisterMessage(BuildSpec.ParametersEntry)

Resources = _reflection.GeneratedProtocolMessageType('Resources', (_message.Message,), {
  'DESCRIPTOR' : _RESOURCES,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.Resources)
  })
_sym_db.RegisterMessage(Resources)

CpuResources = _reflection.GeneratedProtocolMessageType('CpuResources', (_message.Message,), {
  'DESCRIPTOR' : _CPURESOURCES,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.CpuResources)
  })
_sym_db.RegisterMessage(CpuResources)

GpuResources = _reflection.GeneratedProtocolMessageType('GpuResources', (_message.Message,), {
  'DESCRIPTOR' : _GPURESOURCES,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.GpuResources)
  })
_sym_db.RegisterMessage(GpuResources)

GitModelSource = _reflection.GeneratedProtocolMessageType('GitModelSource', (_message.Message,), {
  'DESCRIPTOR' : _GITMODELSOURCE,
  '__module__' : 'qwak.automation.v1.action_pb2'
  # @@protoc_insertion_point(class_scope:qwak.automation.v1.GitModelSource)
  })
_sym_db.RegisterMessage(GitModelSource)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035com.qwak.ai.automation.api.v1P\001'
  _DEPLOYMENTSIZE.fields_by_name['cpu']._options = None
  _DEPLOYMENTSIZE.fields_by_name['cpu']._serialized_options = b'\030\001'
  _DEPLOYMENTSIZE.fields_by_name['memory_amount']._options = None
  _DEPLOYMENTSIZE.fields_by_name['memory_amount']._serialized_options = b'\030\001'
  _DEPLOYMENTSIZE.fields_by_name['memory_units']._options = None
  _DEPLOYMENTSIZE.fields_by_name['memory_units']._serialized_options = b'\030\001'
  _DEPLOYMENTSIZE.fields_by_name['gpu_resources']._options = None
  _DEPLOYMENTSIZE.fields_by_name['gpu_resources']._serialized_options = b'\030\001'
  _BUILDSPEC_PARAMETERSENTRY._options = None
  _BUILDSPEC_PARAMETERSENTRY._serialized_options = b'8\001'
  _RESOURCES.fields_by_name['cpu_resources']._options = None
  _RESOURCES.fields_by_name['cpu_resources']._serialized_options = b'\030\001'
  _RESOURCES.fields_by_name['gpu_resources']._options = None
  _RESOURCES.fields_by_name['gpu_resources']._serialized_options = b'\030\001'
  _GPUTYPE._serialized_start=2938
  _GPUTYPE._serialized_end=3057
  _MEMORYUNIT._serialized_start=3059
  _MEMORYUNIT._serialized_end=3102
  _PURCHASEOPTION._serialized_start=3104
  _PURCHASEOPTION._serialized_end=3145
  _ACTION._serialized_start=252
  _ACTION._serialized_end=405
  _BATCHEXECUTIONACTION._serialized_start=407
  _BATCHEXECUTIONACTION._serialized_end=488
  _BUILDANDDEPLOYACTION._serialized_start=491
  _BUILDANDDEPLOYACTION._serialized_end=696
  _DEPLOYMENTCONDITION._serialized_start=698
  _DEPLOYMENTCONDITION._serialized_end=798
  _BUILDMETRICCONDITION._serialized_start=801
  _BUILDMETRICCONDITION._serialized_end=938
  _DEPLOYMENTSPEC._serialized_start=941
  _DEPLOYMENTSPEC._serialized_end=1164
  _ADVANCEDDEPLOYMENTOPTIONS._serialized_start=1167
  _ADVANCEDDEPLOYMENTOPTIONS._serialized_end=1491
  _DEPLOYMENTSIZE._serialized_start=1494
  _DEPLOYMENTSIZE._serialized_end=1795
  _BUILDSPEC._serialized_start=1798
  _BUILDSPEC._serialized_end=2374
  _BUILDSPEC_PARAMETERSENTRY._serialized_start=2325
  _BUILDSPEC_PARAMETERSENTRY._serialized_end=2374
  _RESOURCES._serialized_start=2377
  _RESOURCES._serialized_end=2626
  _CPURESOURCES._serialized_start=2628
  _CPURESOURCES._serialized_end=2732
  _GPURESOURCES._serialized_start=2734
  _GPURESOURCES._serialized_end=2815
  _GITMODELSOURCE._serialized_start=2817
  _GITMODELSOURCE._serialized_end=2936
# @@protoc_insertion_point(module_scope)



from typing import Optional

from _qwak_proto.qwak.prompt.v1.prompt.prompt_pb2 import (
    Prompt as ProtoPrompt,
    PromptVersion as ProtoPromptVersion,
    PromptVersionDefinition as ProtoPromptVersionDefinition,
    PromptVersionSpec as ProtoPromptVersionSpec,
)
from qwak.clients.prompt_manager.prompt_manager_client import PromptManagerClient
from qwak.clients.prompt_manager.prompt_proto_mapper import PromptProtoMapper
from qwak.exceptions import QwakException
from qwak.llmops.prompt.base import BasePrompt, ChatPrompt, RegisteredPrompt


class PromptManager:
    _prompt_manager_client: PromptManagerClient

    def __init__(self):
        self._prompt_manager_client = PromptManagerClient()

    def register(
        self,
        name: str,
        prompt: BasePrompt,
        prompt_description: Optional[str] = None,
        version_description: Optional[str] = None,
    ) -> RegisteredPrompt:
        """
        Registers a new prompt in Qwak platform. Name must be unique
         and conform to ^[a-z0-9](?:[-_]?[a-z0-9]+)+$
        """
        if not isinstance(prompt, ChatPrompt):
            raise QwakException(f"Got unsupported prompt type: {prompt}")

        version_spec: ProtoPromptVersionSpec = PromptProtoMapper.to_prompt_version_spec(
            version_description=version_description,
            prompt_template=prompt.template,
            model_descriptor=prompt.model,
        )

        registered_prompt: ProtoPrompt = self._prompt_manager_client.create_prompt(
            name=name, prompt_description=prompt_description, version_spec=version_spec
        )

        return PromptProtoMapper.from_prompt(
            name=registered_prompt.name,
            prompt_description=registered_prompt.prompt_spec.description,
            version_description=registered_prompt.default_version_definition.version_spec.description,
            version=registered_prompt.default_version_definition.version_number,
            target_default_version=True,
            prompt_version_definition=registered_prompt.default_version_definition,
        )

    def update(
        self,
        name: str,
        prompt: BasePrompt,
        version_description: Optional[str] = None,
        set_default: bool = False,
    ) -> RegisteredPrompt:
        """
        Creates a new version for an existing prompt, prompt name must already exist.
        `set_default` set to True if this version is to become the default one immediately.
        """
        if not isinstance(prompt, ChatPrompt):
            raise QwakException(f"Got unsupported prompt type: {prompt}")

        version_spec: ProtoPromptVersionSpec = PromptProtoMapper.to_prompt_version_spec(
            version_description=version_description,
            prompt_template=prompt.template,
            model_descriptor=prompt.model,
        )

        prompt_version: ProtoPromptVersion = (
            self._prompt_manager_client.create_prompt_version(
                name=name, version_spec=version_spec, set_default=set_default
            )
        )

        version_for_get_request = (
            None
            if set_default
            else prompt_version.prompt_version_definition.version_number
        )
        return self.get_prompt(name=name, version=version_for_get_request)

    def set_default(self, name: str, version: int):
        """
        Set a version of a registered prompt named: `name`, as the default version
        """
        self._prompt_manager_client.set_default_prompt_version(
            name=name, version=version
        )

    def delete_prompt(self, name: str):
        """
        Delete all version of a prompt, by name
        """
        self._prompt_manager_client.delete_prompt(name=name)

    def delete_prompt_version(self, name: str, version: int):
        """
        Deletes a specific version of a registered prompt
        """
        self._prompt_manager_client.delete_prompt_version(name=name, version=version)

    def get_prompt(self, name: str, version: Optional[int] = None) -> RegisteredPrompt:
        """
        Get a registered prompt by name. To get the default version omit the `version` param, else
        fetch the specified version.
        """

        prompt_default_version: ProtoPrompt = (
            self._prompt_manager_client.get_prompt_by_name(name=name)
        )
        prompt_version_definition: ProtoPromptVersionDefinition = (
            prompt_default_version.default_version_definition
        )
        if version:
            prompt_version: ProtoPromptVersion = (
                self._prompt_manager_client.get_prompt_version_by_name(
                    name=name, version=version
                )
            )
            prompt_version_definition = prompt_version.prompt_version_definition

        return PromptProtoMapper.from_prompt(
            name=prompt_default_version.name,
            prompt_description=prompt_default_version.prompt_spec.description,
            version_description=prompt_version_definition.version_spec.description,
            version=prompt_version_definition.version_number,
            target_default_version=not bool(version),
            prompt_version_definition=prompt_version_definition,
        )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.wrappers_pb2
import qwak.automation.v1.auto_scaling_pb2
import qwak.automation.v1.common_pb2
import qwak.batch_job.v1.batch_job_service_pb2
import qwak.user_application.common.v0.resources_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _GpuType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _GpuTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_GpuType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID: _GpuType.ValueType  # 0
    NVIDIA_K80: _GpuType.ValueType  # 1
    NVIDIA_V100: _GpuType.ValueType  # 2
    NVIDIA_A100: _GpuType.ValueType  # 3
    NVIDIA_T4: _GpuType.ValueType  # 4
    NVIDIA_A10G: _GpuType.ValueType  # 5
    NVIDIA_L4: _GpuType.ValueType  # 6

class GpuType(_GpuType, metaclass=_GpuTypeEnumTypeWrapper): ...

INVALID: GpuType.ValueType  # 0
NVIDIA_K80: GpuType.ValueType  # 1
NVIDIA_V100: GpuType.ValueType  # 2
NVIDIA_A100: GpuType.ValueType  # 3
NVIDIA_T4: GpuType.ValueType  # 4
NVIDIA_A10G: GpuType.ValueType  # 5
NVIDIA_L4: GpuType.ValueType  # 6
global___GpuType = GpuType

class _MemoryUnit:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _MemoryUnitEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_MemoryUnit.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _MemoryUnit.ValueType  # 0
    MIB: _MemoryUnit.ValueType  # 1
    GIB: _MemoryUnit.ValueType  # 2

class MemoryUnit(_MemoryUnit, metaclass=_MemoryUnitEnumTypeWrapper):
    """Should be common"""

UNKNOWN: MemoryUnit.ValueType  # 0
MIB: MemoryUnit.ValueType  # 1
GIB: MemoryUnit.ValueType  # 2
global___MemoryUnit = MemoryUnit

class _PurchaseOption:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _PurchaseOptionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PurchaseOption.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ON_DEMAND: _PurchaseOption.ValueType  # 0
    SPOT: _PurchaseOption.ValueType  # 1

class PurchaseOption(_PurchaseOption, metaclass=_PurchaseOptionEnumTypeWrapper): ...

ON_DEMAND: PurchaseOption.ValueType  # 0
SPOT: PurchaseOption.ValueType  # 1
global___PurchaseOption = PurchaseOption

class Action(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUILD_DEPLOY_FIELD_NUMBER: builtins.int
    BATCH_EXECUTION_FIELD_NUMBER: builtins.int
    @property
    def build_deploy(self) -> global___BuildAndDeployAction: ...
    @property
    def batch_execution(self) -> global___BatchExecutionAction: ...
    def __init__(
        self,
        *,
        build_deploy: global___BuildAndDeployAction | None = ...,
        batch_execution: global___BatchExecutionAction | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["action", b"action", "batch_execution", b"batch_execution", "build_deploy", b"build_deploy"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["action", b"action", "batch_execution", b"batch_execution", "build_deploy", b"build_deploy"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["action", b"action"]) -> typing_extensions.Literal["build_deploy", "batch_execution"] | None: ...

global___Action = Action

class BatchExecutionAction(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXECUTE_BATCH_JOB_FIELD_NUMBER: builtins.int
    @property
    def execute_batch_job(self) -> qwak.batch_job.v1.batch_job_service_pb2.BatchJobRequest: ...
    def __init__(
        self,
        *,
        execute_batch_job: qwak.batch_job.v1.batch_job_service_pb2.BatchJobRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["execute_batch_job", b"execute_batch_job"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["execute_batch_job", b"execute_batch_job"]) -> None: ...

global___BatchExecutionAction = BatchExecutionAction

class BuildAndDeployAction(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUILD_SPEC_FIELD_NUMBER: builtins.int
    DEPLOYMENT_CONDITION_FIELD_NUMBER: builtins.int
    DEPLOYMENT_SPEC_FIELD_NUMBER: builtins.int
    @property
    def build_spec(self) -> global___BuildSpec: ...
    @property
    def deployment_condition(self) -> global___DeploymentCondition: ...
    @property
    def deployment_spec(self) -> global___DeploymentSpec: ...
    def __init__(
        self,
        *,
        build_spec: global___BuildSpec | None = ...,
        deployment_condition: global___DeploymentCondition | None = ...,
        deployment_spec: global___DeploymentSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["build_spec", b"build_spec", "deployment_condition", b"deployment_condition", "deployment_spec", b"deployment_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["build_spec", b"build_spec", "deployment_condition", b"deployment_condition", "deployment_spec", b"deployment_spec"]) -> None: ...

global___BuildAndDeployAction = BuildAndDeployAction

class DeploymentCondition(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUILD_METRIC_FIELD_NUMBER: builtins.int
    @property
    def build_metric(self) -> global___BuildMetricCondition: ...
    def __init__(
        self,
        *,
        build_metric: global___BuildMetricCondition | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["build_metric", b"build_metric", "condition", b"condition"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["build_metric", b"build_metric", "condition", b"condition"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["condition", b"condition"]) -> typing_extensions.Literal["build_metric"] | None: ...

global___DeploymentCondition = DeploymentCondition

class BuildMetricCondition(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    METRIC_NAME_FIELD_NUMBER: builtins.int
    THRESHOLD_DIRECTION_FIELD_NUMBER: builtins.int
    THRESHOLD_FIELD_NUMBER: builtins.int
    metric_name: builtins.str
    """The metric name to check"""
    threshold_direction: qwak.automation.v1.common_pb2.MetricThresholdDirection.ValueType
    """Metric direction to check"""
    threshold: builtins.str
    """The threshold to decide if condition met"""
    def __init__(
        self,
        *,
        metric_name: builtins.str = ...,
        threshold_direction: qwak.automation.v1.common_pb2.MetricThresholdDirection.ValueType = ...,
        threshold: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["metric_name", b"metric_name", "threshold", b"threshold", "threshold_direction", b"threshold_direction"]) -> None: ...

global___BuildMetricCondition = BuildMetricCondition

class DeploymentSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEPLOYMENT_SIZE_FIELD_NUMBER: builtins.int
    ADVANCED_OPTIONS_FIELD_NUMBER: builtins.int
    SELECTED_VARIATION_NAME_FIELD_NUMBER: builtins.int
    ENVIRONMENTS_FIELD_NUMBER: builtins.int
    ENV_VARS_FIELD_NUMBER: builtins.int
    @property
    def deployment_size(self) -> global___DeploymentSize:
        """The deployment"""
    @property
    def advanced_options(self) -> global___AdvancedDeploymentOptions: ...
    selected_variation_name: builtins.str
    """The selected variation name"""
    @property
    def environments(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """The selected environment names"""
    @property
    def env_vars(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Environment variables for the model code"""
    def __init__(
        self,
        *,
        deployment_size: global___DeploymentSize | None = ...,
        advanced_options: global___AdvancedDeploymentOptions | None = ...,
        selected_variation_name: builtins.str = ...,
        environments: collections.abc.Iterable[builtins.str] | None = ...,
        env_vars: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["advanced_options", b"advanced_options", "deployment_size", b"deployment_size"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["advanced_options", b"advanced_options", "deployment_size", b"deployment_size", "env_vars", b"env_vars", "environments", b"environments", "selected_variation_name", b"selected_variation_name"]) -> None: ...

global___DeploymentSpec = DeploymentSpec

class AdvancedDeploymentOptions(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUMBER_OF_HTTP_SERVER_WORKERS_FIELD_NUMBER: builtins.int
    HTTP_REQUEST_TIMEOUT_MS_FIELD_NUMBER: builtins.int
    DAEMON_MODE_FIELD_NUMBER: builtins.int
    CUSTOM_IAM_ROLE_ARN_FIELD_NUMBER: builtins.int
    AUTO_SCALING_CONFIG_FIELD_NUMBER: builtins.int
    MAX_BATCH_SIZE_FIELD_NUMBER: builtins.int
    DEPLOYMENT_PROCESS_TIMEOUT_LIMIT_FIELD_NUMBER: builtins.int
    SERVICE_ACCOUNT_KEY_SECRET_NAME_FIELD_NUMBER: builtins.int
    number_of_http_server_workers: builtins.int
    """Amount of http server workers"""
    http_request_timeout_ms: builtins.int
    """Http request timeout in ms"""
    daemon_mode: builtins.bool
    """Serve model container in daemon mode"""
    custom_iam_role_arn: builtins.str
    """Custom IAM Role ARN"""
    @property
    def auto_scaling_config(self) -> qwak.automation.v1.auto_scaling_pb2.AutoScalingConfig:
        """Auto scaling config"""
    max_batch_size: builtins.int
    """Max batch size of records"""
    deployment_process_timeout_limit: builtins.int
    """The timeout for the deployment (in seconds)"""
    service_account_key_secret_name: builtins.str
    """Custom service account for GCP"""
    def __init__(
        self,
        *,
        number_of_http_server_workers: builtins.int = ...,
        http_request_timeout_ms: builtins.int = ...,
        daemon_mode: builtins.bool = ...,
        custom_iam_role_arn: builtins.str = ...,
        auto_scaling_config: qwak.automation.v1.auto_scaling_pb2.AutoScalingConfig | None = ...,
        max_batch_size: builtins.int = ...,
        deployment_process_timeout_limit: builtins.int = ...,
        service_account_key_secret_name: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["auto_scaling_config", b"auto_scaling_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["auto_scaling_config", b"auto_scaling_config", "custom_iam_role_arn", b"custom_iam_role_arn", "daemon_mode", b"daemon_mode", "deployment_process_timeout_limit", b"deployment_process_timeout_limit", "http_request_timeout_ms", b"http_request_timeout_ms", "max_batch_size", b"max_batch_size", "number_of_http_server_workers", b"number_of_http_server_workers", "service_account_key_secret_name", b"service_account_key_secret_name"]) -> None: ...

global___AdvancedDeploymentOptions = AdvancedDeploymentOptions

class DeploymentSize(google.protobuf.message.Message):
    """Should be common - from deployment manager"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUMBER_OF_PODS_FIELD_NUMBER: builtins.int
    CPU_FIELD_NUMBER: builtins.int
    MEMORY_AMOUNT_FIELD_NUMBER: builtins.int
    MEMORY_UNITS_FIELD_NUMBER: builtins.int
    GPU_RESOURCES_FIELD_NUMBER: builtins.int
    CLIENT_POD_COMPUTE_RESOURCES_FIELD_NUMBER: builtins.int
    number_of_pods: builtins.int
    """Count of pods to deploy."""
    cpu: builtins.float
    """Amount of CPU cores"""
    memory_amount: builtins.int
    """Amount of memory"""
    memory_units: global___MemoryUnit.ValueType
    """Units type of memory"""
    @property
    def gpu_resources(self) -> global___GpuResources:
        """GPU Resource for deployed model"""
    @property
    def client_pod_compute_resources(self) -> qwak.user_application.common.v0.resources_pb2.ClientPodComputeResources:
        """Client pod compute resources"""
    def __init__(
        self,
        *,
        number_of_pods: builtins.int = ...,
        cpu: builtins.float = ...,
        memory_amount: builtins.int = ...,
        memory_units: global___MemoryUnit.ValueType = ...,
        gpu_resources: global___GpuResources | None = ...,
        client_pod_compute_resources: qwak.user_application.common.v0.resources_pb2.ClientPodComputeResources | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["client_pod_compute_resources", b"client_pod_compute_resources", "gpu_resources", b"gpu_resources"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["client_pod_compute_resources", b"client_pod_compute_resources", "cpu", b"cpu", "gpu_resources", b"gpu_resources", "memory_amount", b"memory_amount", "memory_units", b"memory_units", "number_of_pods", b"number_of_pods"]) -> None: ...

global___DeploymentSize = DeploymentSize

class BuildSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class ParametersEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    GIT_MODEL_SOURCE_FIELD_NUMBER: builtins.int
    PARAMETERS_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    BASE_IMAGE_FIELD_NUMBER: builtins.int
    ASSUMED_IAM_ROLE_FIELD_NUMBER: builtins.int
    RESOURCE_FIELD_NUMBER: builtins.int
    MAIN_DIR_FIELD_NUMBER: builtins.int
    DEPENDENCY_FILE_PATH_FIELD_NUMBER: builtins.int
    ENV_VARS_FIELD_NUMBER: builtins.int
    GPU_COMPATIBLE_FIELD_NUMBER: builtins.int
    PUSH_IMAGE_FIELD_NUMBER: builtins.int
    PURCHASE_OPTION_FIELD_NUMBER: builtins.int
    SERVICE_ACCOUNT_KEY_SECRET_NAME_FIELD_NUMBER: builtins.int
    PROVISION_INSTANCE_TIMEOUT_FIELD_NUMBER: builtins.int
    @property
    def git_model_source(self) -> global___GitModelSource:
        """Git Details to get model from."""
    @property
    def parameters(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """Parameters for the build"""
    @property
    def tags(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Tags to save on the build"""
    base_image: builtins.str
    assumed_iam_role: builtins.str
    @property
    def resource(self) -> global___Resources: ...
    main_dir: builtins.str
    """The main dir of the model"""
    dependency_file_path: builtins.str
    """The custom dependency file path."""
    @property
    def env_vars(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Environment variables for the model code"""
    gpu_compatible: builtins.bool
    """If the model is GPU compatible"""
    @property
    def push_image(self) -> google.protobuf.wrappers_pb2.BoolValue: ...
    purchase_option: global___PurchaseOption.ValueType
    service_account_key_secret_name: builtins.str
    """Custom service account for GCP"""
    provision_instance_timeout: builtins.int
    def __init__(
        self,
        *,
        git_model_source: global___GitModelSource | None = ...,
        parameters: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        tags: collections.abc.Iterable[builtins.str] | None = ...,
        base_image: builtins.str = ...,
        assumed_iam_role: builtins.str = ...,
        resource: global___Resources | None = ...,
        main_dir: builtins.str = ...,
        dependency_file_path: builtins.str = ...,
        env_vars: collections.abc.Iterable[builtins.str] | None = ...,
        gpu_compatible: builtins.bool = ...,
        push_image: google.protobuf.wrappers_pb2.BoolValue | None = ...,
        purchase_option: global___PurchaseOption.ValueType = ...,
        service_account_key_secret_name: builtins.str = ...,
        provision_instance_timeout: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["git_model_source", b"git_model_source", "push_image", b"push_image", "resource", b"resource"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["assumed_iam_role", b"assumed_iam_role", "base_image", b"base_image", "dependency_file_path", b"dependency_file_path", "env_vars", b"env_vars", "git_model_source", b"git_model_source", "gpu_compatible", b"gpu_compatible", "main_dir", b"main_dir", "parameters", b"parameters", "provision_instance_timeout", b"provision_instance_timeout", "purchase_option", b"purchase_option", "push_image", b"push_image", "resource", b"resource", "service_account_key_secret_name", b"service_account_key_secret_name", "tags", b"tags"]) -> None: ...

global___BuildSpec = BuildSpec

class Resources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CPU_RESOURCES_FIELD_NUMBER: builtins.int
    GPU_RESOURCES_FIELD_NUMBER: builtins.int
    CLIENT_POD_COMPUTE_RESOURCES_FIELD_NUMBER: builtins.int
    @property
    def cpu_resources(self) -> global___CpuResources:
        """CPU Properties"""
    @property
    def gpu_resources(self) -> global___GpuResources:
        """GPU Properties"""
    @property
    def client_pod_compute_resources(self) -> qwak.user_application.common.v0.resources_pb2.ClientPodComputeResources:
        """Client pod compute resources"""
    def __init__(
        self,
        *,
        cpu_resources: global___CpuResources | None = ...,
        gpu_resources: global___GpuResources | None = ...,
        client_pod_compute_resources: qwak.user_application.common.v0.resources_pb2.ClientPodComputeResources | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["client_pod_compute_resources", b"client_pod_compute_resources", "cpu_resources", b"cpu_resources", "gpu_resources", b"gpu_resources", "resource", b"resource"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["client_pod_compute_resources", b"client_pod_compute_resources", "cpu_resources", b"cpu_resources", "gpu_resources", b"gpu_resources", "resource", b"resource"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["resource", b"resource"]) -> typing_extensions.Literal["cpu_resources", "gpu_resources", "client_pod_compute_resources"] | None: ...

global___Resources = Resources

class CpuResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CPU_FIELD_NUMBER: builtins.int
    MEMORY_AMOUNT_FIELD_NUMBER: builtins.int
    MEMORY_UNITS_FIELD_NUMBER: builtins.int
    cpu: builtins.float
    """Amount of CPU cores"""
    memory_amount: builtins.int
    """Amount of memory"""
    memory_units: global___MemoryUnit.ValueType
    """Units type of memory"""
    def __init__(
        self,
        *,
        cpu: builtins.float = ...,
        memory_amount: builtins.int = ...,
        memory_units: global___MemoryUnit.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["cpu", b"cpu", "memory_amount", b"memory_amount", "memory_units", b"memory_units"]) -> None: ...

global___CpuResources = CpuResources

class GpuResources(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GPU_TYPE_FIELD_NUMBER: builtins.int
    GPU_AMOUNT_FIELD_NUMBER: builtins.int
    gpu_type: global___GpuType.ValueType
    """The type of the GPU"""
    gpu_amount: builtins.int
    """Amount of GPUs"""
    def __init__(
        self,
        *,
        gpu_type: global___GpuType.ValueType = ...,
        gpu_amount: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["gpu_amount", b"gpu_amount", "gpu_type", b"gpu_type"]) -> None: ...

global___GpuResources = GpuResources

class GitModelSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GIT_URI_FIELD_NUMBER: builtins.int
    GIT_CREDENTIALS_SECRET_NAME_FIELD_NUMBER: builtins.int
    GIT_BRANCH_FIELD_NUMBER: builtins.int
    GIT_SSH_SECRET_NAME_FIELD_NUMBER: builtins.int
    git_uri: builtins.str
    """git uri to fetch models code from"""
    git_credentials_secret_name: builtins.str
    """Secret to fetch git credentials, overrides git_credentials if set"""
    git_branch: builtins.str
    git_ssh_secret_name: builtins.str
    """Ssh keu to fetch git credentials, overrides git_secret_ssh"""
    def __init__(
        self,
        *,
        git_uri: builtins.str = ...,
        git_credentials_secret_name: builtins.str = ...,
        git_branch: builtins.str = ...,
        git_ssh_secret_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["git_branch", b"git_branch", "git_credentials_secret_name", b"git_credentials_secret_name", "git_ssh_secret_name", b"git_ssh_secret_name", "git_uri", b"git_uri"]) -> None: ...

global___GitModelSource = GitModelSource



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from _qwak_proto.qwak.execution.v1.state import execution_state_service_pb2 as qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2


class FeatureStoreExecutionStateServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetFeaturesetStateById = channel.unary_unary(
                '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/GetFeaturesetStateById',
                request_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByIdRequest.SerializeToString,
                response_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByIdResponse.FromString,
                )
        self.GetFeaturesetStateByName = channel.unary_unary(
                '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/GetFeaturesetStateByName',
                request_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByNameRequest.SerializeToString,
                response_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByNameResponse.FromString,
                )
        self.GetMultiFeaturesetStateByIds = channel.unary_unary(
                '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/GetMultiFeaturesetStateByIds',
                request_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByIdsRequest.SerializeToString,
                response_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByIdsResponse.FromString,
                )
        self.GetMultiFeaturesetStateByName = channel.unary_unary(
                '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/GetMultiFeaturesetStateByName',
                request_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByNameRequest.SerializeToString,
                response_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByNameResponse.FromString,
                )
        self.UpdateMultiExecutionState = channel.unary_unary(
                '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/UpdateMultiExecutionState',
                request_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.UpdateMultiExecutionStateRequest.SerializeToString,
                response_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.UpdateMultiExecutionStateResponse.FromString,
                )
        self.DeleteFeaturesetStateById = channel.unary_unary(
                '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/DeleteFeaturesetStateById',
                request_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.DeleteFeaturesetStateByIdRequest.SerializeToString,
                response_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.DeleteFeaturesetStateByIdResponse.FromString,
                )


class FeatureStoreExecutionStateServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetFeaturesetStateById(self, request, context):
        """Get feature set state by feature set id
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFeaturesetStateByName(self, request, context):
        """Get feature set state by feature set name
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetMultiFeaturesetStateByIds(self, request, context):
        """Get state of multiple feature sets by a list of IDs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetMultiFeaturesetStateByName(self, request, context):
        """Get state of multiple feature sets by a list of names
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateMultiExecutionState(self, request, context):
        """Update state multiple executions - called by the edge state publisher
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteFeaturesetStateById(self, request, context):
        """Delete feature set state
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_FeatureStoreExecutionStateServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetFeaturesetStateById': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeaturesetStateById,
                    request_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByIdRequest.FromString,
                    response_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByIdResponse.SerializeToString,
            ),
            'GetFeaturesetStateByName': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFeaturesetStateByName,
                    request_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByNameRequest.FromString,
                    response_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByNameResponse.SerializeToString,
            ),
            'GetMultiFeaturesetStateByIds': grpc.unary_unary_rpc_method_handler(
                    servicer.GetMultiFeaturesetStateByIds,
                    request_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByIdsRequest.FromString,
                    response_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByIdsResponse.SerializeToString,
            ),
            'GetMultiFeaturesetStateByName': grpc.unary_unary_rpc_method_handler(
                    servicer.GetMultiFeaturesetStateByName,
                    request_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByNameRequest.FromString,
                    response_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByNameResponse.SerializeToString,
            ),
            'UpdateMultiExecutionState': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateMultiExecutionState,
                    request_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.UpdateMultiExecutionStateRequest.FromString,
                    response_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.UpdateMultiExecutionStateResponse.SerializeToString,
            ),
            'DeleteFeaturesetStateById': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteFeaturesetStateById,
                    request_deserializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.DeleteFeaturesetStateByIdRequest.FromString,
                    response_serializer=qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.DeleteFeaturesetStateByIdResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class FeatureStoreExecutionStateService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetFeaturesetStateById(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/GetFeaturesetStateById',
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByIdRequest.SerializeToString,
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByIdResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFeaturesetStateByName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/GetFeaturesetStateByName',
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByNameRequest.SerializeToString,
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetFeaturesetStateByNameResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetMultiFeaturesetStateByIds(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/GetMultiFeaturesetStateByIds',
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByIdsRequest.SerializeToString,
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByIdsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetMultiFeaturesetStateByName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/GetMultiFeaturesetStateByName',
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByNameRequest.SerializeToString,
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.GetMultiFeaturesetStateByNameResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateMultiExecutionState(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/UpdateMultiExecutionState',
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.UpdateMultiExecutionStateRequest.SerializeToString,
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.UpdateMultiExecutionStateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteFeaturesetStateById(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.feature.store.execution.v1.state.FeatureStoreExecutionStateService/DeleteFeaturesetStateById',
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.DeleteFeaturesetStateByIdRequest.SerializeToString,
            qwak_dot_execution_dot_v1_dot_state_dot_execution__state__service__pb2.DeleteFeaturesetStateByIdResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from dependency_injector import containers, providers
from qwak.model.decorators.impl.api_implementation import create_decorator_function
from qwak.model.decorators.impl.timer_implementation import create_qwak_timer


class QwakRuntimeContainer(containers.DeclarativeContainer):
    """
    Qwak Core's Runtime Dependency Injection Container
    """

    api_decorator_function_creator = providers.Object(create_decorator_function)
    timer_function_creator = providers.Object(create_qwak_timer)


class ContainerLock:
    locked: bool = False



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/audience/v1/audience_api.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.audience.v1 import audience_pb2 as qwak_dot_audience_dot_v1_dot_audience__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#qwak/audience/v1/audience_api.proto\x12\x10qwak.audience.v1\x1a\x1fqwak/audience/v1/audience.proto\"E\n\x15\x43reateAudienceRequest\x12,\n\x08\x61udience\x18\x01 \x01(\x0b\x32\x1a.qwak.audience.v1.Audience\"-\n\x16\x43reateAudienceResponse\x12\x13\n\x0b\x61udience_id\x18\x01 \x01(\t\"Q\n\x15UpdateAudienceRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12,\n\x08\x61udience\x18\x03 \x01(\x0b\x32\x1a.qwak.audience.v1.Audience\"\x18\n\x16UpdateAudienceResponse\")\n\x12GetAudienceRequest\x12\x13\n\x0b\x61udience_id\x18\x01 \x01(\t\"C\n\x13GetAudienceResponse\x12,\n\x08\x61udience\x18\x01 \x01(\x0b\x32\x1a.qwak.audience.v1.Audience\"\x15\n\x13ListAudienceRequest\"Q\n\x14ListAudienceResponse\x12\x39\n\x10\x61udience_entries\x18\x01 \x03(\x0b\x32\x1f.qwak.audience.v1.AudienceEntry\",\n\x15\x44\x65leteAudienceRequest\x12\x13\n\x0b\x61udience_id\x18\x01 \x01(\t\"\x18\n\x16\x44\x65leteAudienceResponse\".\n\x14SyncAudiencesRequest\x12\x16\n\x0e\x65nvironment_id\x18\x01 \x01(\t\"\x17\n\x15SyncAudiencesResponse2\xd9\x04\n\x0b\x41udienceAPI\x12\x63\n\x0e\x43reateAudience\x12\'.qwak.audience.v1.CreateAudienceRequest\x1a(.qwak.audience.v1.CreateAudienceResponse\x12\x63\n\x0eUpdateAudience\x12\'.qwak.audience.v1.UpdateAudienceRequest\x1a(.qwak.audience.v1.UpdateAudienceResponse\x12Z\n\x0bGetAudience\x12$.qwak.audience.v1.GetAudienceRequest\x1a%.qwak.audience.v1.GetAudienceResponse\x12]\n\x0cListAudience\x12%.qwak.audience.v1.ListAudienceRequest\x1a&.qwak.audience.v1.ListAudienceResponse\x12\x63\n\x0e\x44\x65leteAudience\x12\'.qwak.audience.v1.DeleteAudienceRequest\x1a(.qwak.audience.v1.DeleteAudienceResponse\x12`\n\rSyncAudiences\x12&.qwak.audience.v1.SyncAudiencesRequest\x1a\'.qwak.audience.v1.SyncAudiencesResponseBE\n\x14\x63om.qwak.audience.v1B\x10\x41udienceAPIProtoP\x01Z\x19qwak/audience/v1;audienceb\x06proto3')



_CREATEAUDIENCEREQUEST = DESCRIPTOR.message_types_by_name['CreateAudienceRequest']
_CREATEAUDIENCERESPONSE = DESCRIPTOR.message_types_by_name['CreateAudienceResponse']
_UPDATEAUDIENCEREQUEST = DESCRIPTOR.message_types_by_name['UpdateAudienceRequest']
_UPDATEAUDIENCERESPONSE = DESCRIPTOR.message_types_by_name['UpdateAudienceResponse']
_GETAUDIENCEREQUEST = DESCRIPTOR.message_types_by_name['GetAudienceRequest']
_GETAUDIENCERESPONSE = DESCRIPTOR.message_types_by_name['GetAudienceResponse']
_LISTAUDIENCEREQUEST = DESCRIPTOR.message_types_by_name['ListAudienceRequest']
_LISTAUDIENCERESPONSE = DESCRIPTOR.message_types_by_name['ListAudienceResponse']
_DELETEAUDIENCEREQUEST = DESCRIPTOR.message_types_by_name['DeleteAudienceRequest']
_DELETEAUDIENCERESPONSE = DESCRIPTOR.message_types_by_name['DeleteAudienceResponse']
_SYNCAUDIENCESREQUEST = DESCRIPTOR.message_types_by_name['SyncAudiencesRequest']
_SYNCAUDIENCESRESPONSE = DESCRIPTOR.message_types_by_name['SyncAudiencesResponse']
CreateAudienceRequest = _reflection.GeneratedProtocolMessageType('CreateAudienceRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEAUDIENCEREQUEST,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.CreateAudienceRequest)
  })
_sym_db.RegisterMessage(CreateAudienceRequest)

CreateAudienceResponse = _reflection.GeneratedProtocolMessageType('CreateAudienceResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEAUDIENCERESPONSE,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.CreateAudienceResponse)
  })
_sym_db.RegisterMessage(CreateAudienceResponse)

UpdateAudienceRequest = _reflection.GeneratedProtocolMessageType('UpdateAudienceRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEAUDIENCEREQUEST,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.UpdateAudienceRequest)
  })
_sym_db.RegisterMessage(UpdateAudienceRequest)

UpdateAudienceResponse = _reflection.GeneratedProtocolMessageType('UpdateAudienceResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEAUDIENCERESPONSE,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.UpdateAudienceResponse)
  })
_sym_db.RegisterMessage(UpdateAudienceResponse)

GetAudienceRequest = _reflection.GeneratedProtocolMessageType('GetAudienceRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETAUDIENCEREQUEST,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.GetAudienceRequest)
  })
_sym_db.RegisterMessage(GetAudienceRequest)

GetAudienceResponse = _reflection.GeneratedProtocolMessageType('GetAudienceResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETAUDIENCERESPONSE,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.GetAudienceResponse)
  })
_sym_db.RegisterMessage(GetAudienceResponse)

ListAudienceRequest = _reflection.GeneratedProtocolMessageType('ListAudienceRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTAUDIENCEREQUEST,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.ListAudienceRequest)
  })
_sym_db.RegisterMessage(ListAudienceRequest)

ListAudienceResponse = _reflection.GeneratedProtocolMessageType('ListAudienceResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTAUDIENCERESPONSE,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.ListAudienceResponse)
  })
_sym_db.RegisterMessage(ListAudienceResponse)

DeleteAudienceRequest = _reflection.GeneratedProtocolMessageType('DeleteAudienceRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEAUDIENCEREQUEST,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.DeleteAudienceRequest)
  })
_sym_db.RegisterMessage(DeleteAudienceRequest)

DeleteAudienceResponse = _reflection.GeneratedProtocolMessageType('DeleteAudienceResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEAUDIENCERESPONSE,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.DeleteAudienceResponse)
  })
_sym_db.RegisterMessage(DeleteAudienceResponse)

SyncAudiencesRequest = _reflection.GeneratedProtocolMessageType('SyncAudiencesRequest', (_message.Message,), {
  'DESCRIPTOR' : _SYNCAUDIENCESREQUEST,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.SyncAudiencesRequest)
  })
_sym_db.RegisterMessage(SyncAudiencesRequest)

SyncAudiencesResponse = _reflection.GeneratedProtocolMessageType('SyncAudiencesResponse', (_message.Message,), {
  'DESCRIPTOR' : _SYNCAUDIENCESRESPONSE,
  '__module__' : 'qwak.audience.v1.audience_api_pb2'
  # @@protoc_insertion_point(class_scope:qwak.audience.v1.SyncAudiencesResponse)
  })
_sym_db.RegisterMessage(SyncAudiencesResponse)

_AUDIENCEAPI = DESCRIPTOR.services_by_name['AudienceAPI']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024com.qwak.audience.v1B\020AudienceAPIProtoP\001Z\031qwak/audience/v1;audience'
  _CREATEAUDIENCEREQUEST._serialized_start=90
  _CREATEAUDIENCEREQUEST._serialized_end=159
  _CREATEAUDIENCERESPONSE._serialized_start=161
  _CREATEAUDIENCERESPONSE._serialized_end=206
  _UPDATEAUDIENCEREQUEST._serialized_start=208
  _UPDATEAUDIENCEREQUEST._serialized_end=289
  _UPDATEAUDIENCERESPONSE._serialized_start=291
  _UPDATEAUDIENCERESPONSE._serialized_end=315
  _GETAUDIENCEREQUEST._serialized_start=317
  _GETAUDIENCEREQUEST._serialized_end=358
  _GETAUDIENCERESPONSE._serialized_start=360
  _GETAUDIENCERESPONSE._serialized_end=427
  _LISTAUDIENCEREQUEST._serialized_start=429
  _LISTAUDIENCEREQUEST._serialized_end=450
  _LISTAUDIENCERESPONSE._serialized_start=452
  _LISTAUDIENCERESPONSE._serialized_end=533
  _DELETEAUDIENCEREQUEST._serialized_start=535
  _DELETEAUDIENCEREQUEST._serialized_end=579
  _DELETEAUDIENCERESPONSE._serialized_start=581
  _DELETEAUDIENCERESPONSE._serialized_end=605
  _SYNCAUDIENCESREQUEST._serialized_start=607
  _SYNCAUDIENCESREQUEST._serialized_end=653
  _SYNCAUDIENCESRESPONSE._serialized_start=655
  _SYNCAUDIENCESRESPONSE._serialized_end=678
  _AUDIENCEAPI._serialized_start=681
  _AUDIENCEAPI._serialized_end=1282
# @@protoc_insertion_point(module_scope)



import re
from pathlib import Path
from typing import Union

from qwak.inner.build_logic.interface.build_logger_interface import BuildLogger

_GIT_URI_REGEX = re.compile(r"^[^/]*:")
_FILE_URI_REGEX = re.compile(r"^file://.+")
_ZIP_URI_REGEX = re.compile(r".+\.zip")


def is_zip_uri(uri: Union[str, Path]) -> bool:
    return uri and _ZIP_URI_REGEX.match(str(uri)) is not None


def is_local_dir(uri: Union[str, Path]) -> bool:
    return uri and not _GIT_URI_REGEX.match(str(uri)) and Path(uri).is_dir()


def is_git_uri(uri: Union[str, Path]) -> bool:
    return uri and _GIT_URI_REGEX.match(str(uri)) is not None


def get_git_commit_id(path: Union[str, Path], build_logger: BuildLogger) -> str:
    try:
        import git

        repo = git.Repo(path=path, search_parent_directories=True)
        return repo.head.object.hexsha
    except Exception as e:
        build_logger.warning(f"Failed to get git commit with error: {e}")
        return ""



from collections import defaultdict
from datetime import date, datetime
from functools import reduce
from typing import DefaultDict, Dict, List, Tuple, Union

from qwak.clients.administration.eco_system.client import EcosystemClient

try:
    import pandas as pd
except ImportError:
    pass

from warnings import warn

from dateutil.parser import ParserError
from qwak.exceptions import QwakException
from qwak.feature_store._common.functions import normalize_cols
from qwak.feature_store.offline.athena.athena_query_engine import AthenaQueryEngine


class OfflineClient:
    """
    A class used to retrieve data from the offline store - mainly used to get train data for models.
    It requires qwak configure and aws access.
    @deprecated
    """

    # Default SCD parameters of the feature store etl
    ANALYTICS_DB_PREFIX = "qwak_analytics_feature_store"
    FS_DB_PREFIX = "qwak_feature_store"
    FS_TABLE_NAME_PREFIX = "offline_feature_store"
    FS_START_TIME_COLUMN = "start_timestamp"
    FS_END_TIME_COLUMN = "end_timestamp"
    FEATURE_STORE_PREFIX = "feature_store"

    DEFAULT_NUMBER_OF_SAMPLE_DATA_ROWS = "100"

    def __init__(
        self,
        query_engine=None,
        environment_id=None,
    ):
        warn(
            "This Client will be deprecated soon, Please use OfflineClientV2 Instead",
            DeprecationWarning,
            stacklevel=2,
        )
        self.query_engine = (
            query_engine if query_engine is not None else AthenaQueryEngine()
        )
        self.quotes = self.query_engine.get_quotes()
        if environment_id is None:
            user_context = EcosystemClient().get_authenticated_user_context().user
            environment_id = (
                user_context.account_details.default_environment_id.replace("-", "_")
            )

        self.environment_id = environment_id.replace("-", "_")
        self.FS_DB_NAME = self.FS_DB_PREFIX + "_" + self.environment_id
        self.FS_ANALYTICS_DB_NAME = self.ANALYTICS_DB_PREFIX + "_" + self.environment_id

    def get_feature_range_values(
        self,
        entity_key_to_features: dict,
        start_date: Union[datetime, date],
        end_date: Union[datetime, date],
    ):
        """
        :param entity_key_to_features: a dictionary { entity_key(s) -> features list }.
        :param start_date: the column name of the point in time column (default - timestamp)
        :param end_date: the column name of the point in time column (default - timestamp)
        :return: a pandas dataframe or a list of dataframes (a dataframe for every entity_key) - all feature values for
         all entites under the given date range
        @depracted

        each row in the returned data-frame is constructed by retrieving the requested features of the entity
         key(s) for all entity values in within the defined date tange.

        Feature sets should be named [Feature Set Name].[Feature Name],
        i.e: user_purchases.number_of_purchases.

        Examples:
        >>> from datetime import datetime
        >>> from qwak.feature_store.offline import OfflineClient
        >>>
        >>> start_date = datetime(year=2021, month=1, day=1)
        >>> end_date = datetime(year=2021, month=1, day=3)
        >>>
        >>> key_to_features = {'uuid': ['user_purchases.number_of_purchases',
        >>>                             'user_purchases.avg_purchase_amount']}
        >>>
        >>> offline_feature_store = OfflineClient()
        >>>
        >>> train_df = offline_feature_store.get_feature_range_values(
        >>>                entity_key_to_features=key_to_features,
        >>>                start_date=start_date,
        >>>                end_date=end_date)
        >>>
        >>> print(train_df.head())
        >>> #	     uuid	         timestamp	      user_purchases.number_of_purchases	user_purchases.avg_purchase_amount
        >>> # 0	      1	        2021-01-02 17:00:00	                 76	                                4.796842
        >>> # 1	      1	        2021-01-01 12:00:00	                 5	                                1.548000
        >>> # 2	      2	        2021-01-02 12:00:00	                 5	                                5.548000
        >>> # 3	      2	        2021-01-01 18:00:00	                 5	                                2.788000
        """
        warn(
            "This method will be deprecated soon, Please use OfflineClientV2: get_feature_values() Instead",
            DeprecationWarning,
            stacklevel=2,
        )
        try:
            from qwak.feature_store._common.featureset_asterisk_handler import (
                unpack_asterisk_features_from_key_mapping,
            )

            entity_key_to_features = unpack_asterisk_features_from_key_mapping(
                entity_key_to_features, lambda: self
            )
            self._validate_range_query_inputs(
                entity_key_to_features, start_date, end_date
            )

            feature_set_name_to_feature_list = (
                self._partition_feature_set_by_entity_feature(entity_key_to_features)
            )
            feature_set_to_dtypes = self._validate_database_and_get_feature_set_dtypes(
                feature_set_name_to_feature_list
            )
            self._validate_features_exist(
                feature_set_name_to_feature_list, feature_set_to_dtypes
            )

            df = self._run_ranges_query(
                feature_set_name_to_feature_list, start_date, end_date
            )

            return self._normalize_df(df)

        except QwakException as qwak_exception:
            raise QwakException(
                f"Got the following Qwak generated exception: {qwak_exception}"
            )
        except Exception as e:
            raise QwakException(f"Got the following run-time exception: {e}")
        finally:
            try:
                self.query_engine.cleanup()
            except Exception as e:
                print(f"Got the following run-time exception during cleanup: {e}")

    def get_feature_values(
        self,
        entity_key_to_features: dict,
        population: "pd.DataFrame",
        point_in_time_column_name: str = "timestamp",
    ):
        """
        :param entity_key_to_features: a dictionary { entity_key(s) -> features list }.
        :param population: a pandas data-frame with a point in time column
                           and a column for each entity key defined at entity_key_to_features.
        :param point_in_time_column_name: the column name of the point in time column (default - timestamp)
        :return: a pandas data-frame - the population joined with the feature values for all
                                       the requested entities and features.
        @deprecated

        each row in the returned data-frame is constructed by retrieving the requested features of the entity key(s) for
        the specific entity value(s) in the population and on the specific point in time defined.

        Feature sets should be named [Feature Set Name].[Feature Name],
        i.e: user_purchases.number_of_purchases.

        Examples:
        >>> import pandas as pd
        >>> from qwak.feature_store.offline import OfflineClient
        >>>
        >>> population_df = pd.DataFrame(
        >>>     columns= ['uuid',       'timestamp'     ],
        >>>     data   =[[ '1'  , '2021-01-02 17:00:00' ],
        >>>              [ '2'  , '2021-01-01 12:00:00' ]])
        >>>
        >>> key_to_features = {'uuid': ['user_purchases.number_of_purchases',
        >>>                             'user_purchases.avg_purchase_amount']}
        >>>
        >>> offline_feature_store = OfflineClient()
        >>>
        >>> train_df = offline_feature_store.get_feature_values(
        >>>                entity_key_to_features=key_to_features,
        >>>                population=population_df,
        >>>                point_in_time_column_name='timestamp')
        >>>
        >>> print(train_df.head())
        >>> #	     uuid	         timestamp	      user_purchases.number_of_purchases	user_purchases.avg_purchase_amount
        >>> # 0	      1	        2021-04-24 17:00:00	                 76	                                4.796842
        >>> # 1	      2	        2021-04-24 12:00:00	                 5	                                1.548000
        """
        warn(
            "This method will be deprecated soon, Please use OfflineClientV2: get_feature_range_values() Instead",
            DeprecationWarning,
            stacklevel=2,
        )
        import pandas as pd

        try:
            from qwak.feature_store._common.featureset_asterisk_handler import (
                unpack_asterisk_features_from_key_mapping,
            )

            population = population.copy()

            entity_key_to_features = unpack_asterisk_features_from_key_mapping(
                entity_key_to_features, lambda: self
            )

            self._validate_point_in_time_query_inputs(
                entity_key_to_features, population, point_in_time_column_name
            )

            feature_set_name_to_feature_dict = (
                self._partition_feature_set_by_entity_feature(entity_key_to_features)
            )

            feature_set_to_dtypes = self._validate_database_and_get_feature_set_dtypes(
                feature_set_name_to_feature_dict
            )

            self._validate_features_exist(
                feature_set_name_to_feature_dict,
                feature_set_to_dtypes,
            )

            population = self._align_entity_key_dtype(
                feature_set_to_dtypes, entity_key_to_features, population
            )

            uploaded_population_path = self.query_engine.upload_table(population)

            df = pd.DataFrame()

            if feature_set_name_to_feature_dict:
                df = self._run_point_in_time_query(
                    feature_set_name_to_feature_dict,
                    uploaded_population_path,
                    point_in_time_column_name,
                    [column.lower() for column in population.columns],
                )

            return self._normalize_df(df)

        except QwakException as qwak_exception:
            raise QwakException(
                f"Got the following Qwak generated exception: {qwak_exception}"
            )
        except Exception as e:
            raise QwakException(f"Got the following run-time exception: {e}")
        finally:
            try:
                self.query_engine.cleanup()
            except Exception as e:
                print(f"Got the following run-time exception during cleanup: {e}")

    @staticmethod
    def _normalize_df(df: "pd.DataFrame") -> "pd.DataFrame":
        columns = df.columns.values.tolist()
        new_columns = normalize_cols(columns)
        df.columns = new_columns
        return df

    @staticmethod
    def _validate_range_query_inputs(
        entity_key_to_features: dict, start_date: datetime, end_date: datetime
    ):
        missing_features_entity_keys = [
            entity_key
            for entity_key, features in entity_key_to_features.items()
            if not features
        ]

        if missing_features_entity_keys:
            raise QwakException(
                f"Features of an entity key must exist, missing features for: [{missing_features_entity_keys}]"
            )

        if (end_date - start_date).total_seconds() < 0:
            raise QwakException("Invalid date range - end date is before start date")

    @staticmethod
    def _validate_point_in_time_query_inputs(
        entity_key_to_features: dict,
        population: "pd.DataFrame",
        point_in_time_column_name: str,
    ):
        """
        Validates that the entity keys, timestamp cols and features exist in DB
        """
        missing_keys = [
            entity_key
            for entity_key in entity_key_to_features.keys()
            if entity_key not in population
        ]
        if missing_keys:
            raise QwakException(
                f"The entity keys must be in population_df columns, missing: [{missing_keys}]"
            )

        missing_features_entity_keys = [
            entity_key
            for entity_key, features in entity_key_to_features.items()
            if not features
        ]

        if missing_features_entity_keys:
            raise QwakException(
                f"Features of an entity key must exist, missing features for: [{missing_features_entity_keys}]"
            )

        if point_in_time_column_name not in population:
            raise QwakException(
                "The point in time column must be part of the population dataframe"
            )

        from pandas.api.types import is_datetime64_any_dtype

        if not is_datetime64_any_dtype(population[point_in_time_column_name]):
            try:
                population[point_in_time_column_name] = pd.to_datetime(
                    population[point_in_time_column_name]
                )
            except ParserError as e:
                raise QwakException(
                    f"It was not possible to cast provided point in time column to datetime"
                    f"\nError message: {e}"
                )

    @staticmethod
    def _partition_feature_set_by_entity_feature(
        entity_key_to_features,
    ) -> DefaultDict[Tuple[str, str], List[str]]:
        """
        Partition feature by entity key and featureset name
        Args:
            entity_key_to_features: dict of entity_key -> full feature name
        Returns:
            dict of (entity_key,featureset_name) -> list of feature names
        """
        feature_name_to_feature_list = defaultdict(list)

        for entity_key, feature_list in entity_key_to_features.items():
            for feature in feature_list:
                split_feature_set_and_feature = feature.lower().split(".")
                if len(split_feature_set_and_feature) != 2:
                    raise QwakException(
                        f"Failed to verify features. Name should be: <feature set name>.<feature name>. "
                        f"Current name is: {feature}"
                    )
                feature_set_name = split_feature_set_and_feature[0]
                feature_name_to_feature_list[(entity_key, feature_set_name)].append(
                    feature
                )

        return feature_name_to_feature_list

    def _validate_database_and_get_feature_set_dtypes(
        self, feature_name_to_feature_list: DefaultDict[Tuple[str, str], List[str]]
    ) -> Dict[Tuple[str, str], List[Tuple[str, str]]]:
        """
        Args:
            feature_name_to_feature_list:  dictionary from feature set name to its' features

        Returns
            dictionary from feature set name and entity key to a list of feature name, feature type
        """
        if self.FS_DB_NAME not in self._fs_db_names():
            raise QwakException("Offline feature store does not contain any data")

        feature_set_to_dtypes = {}
        for (
            entity_key,
            feature_set_name,
        ), feature_list in feature_name_to_feature_list.items():
            table_name = self._get_offline_feature_store_full_name(feature_set_name)
            if table_name not in self._fs_tables_names():
                raise QwakException(
                    f"[{feature_set_name}] feature set does not contain any data"
                )

            columns_query_result = self.query_engine.run_query(
                f"SELECT * FROM INFORMATION_SCHEMA.COLUMNS "  # nosec B608
                f"WHERE TABLE_SCHEMA = '{self.FS_DB_NAME}' "
                f"AND TABLE_NAME = '{table_name}'"
            )

            feature_set_to_dtypes[(entity_key, feature_set_name)] = [
                (column_tup[3], column_tup[7]) for column_tup in columns_query_result
            ]
        return feature_set_to_dtypes

    @staticmethod
    def _validate_features_exist(
        feature_name_to_feature_list: defaultdict,
        feature_set_to_dtypes: Dict[Tuple[str, str], List[Tuple[str, str]]],
    ):
        """
        Args:
            feature_name_to_feature_list: dictionary from feature set name to its' features
            feature_set_to_dtypes: dictionary from feature set name and entity key
                                   to a list of feature name, feature type
        """
        for (
            entity_key,
            feature_set_name,
        ), feature_list in feature_name_to_feature_list.items():
            columns = [
                column_tuple[0].lower()
                for column_tuple in feature_set_to_dtypes[
                    (entity_key, feature_set_name)
                ]
            ]
            absent_features = [
                feature for feature in feature_list if feature not in columns
            ]
            if absent_features:
                raise QwakException(
                    f"Missing the following features for the feature set "
                    f"[{feature_set_name}]:"
                    f"\n{absent_features}"
                )

    def _align_entity_key_dtype(
        self,
        feature_set_to_dtypes: Dict[Tuple[str, str], List[Tuple[str, str]]],
        entity_key_to_features: Dict[str, List[str]],
        population: "pd.DataFrame",
    ) -> "pd.DataFrame":
        """
        Args:
            feature_set_to_dtypes: dictionary from feature set name and entity key
                                   to a list of feature name, feature type
            entity_key_to_features: a dictionary { entity_key(s) -> features list }.
            population: a pandas data-frame with a point in time column
                           and a column for each entity key defined at entity_key_to_features.
        Returns:
            entity type aligned population df
        """
        sql_type_to_pandas = {
            "string": "string",
            "integer": "int",
            "varchar": "string",
            "text": "string",
            "bigint": "int",
        }

        entity_key_to_dtype = self._validate_and_get_entity_keys_dtypes(
            entity_key_to_features, population
        )

        for (
            entity_key,
            feature_set_name,
        ), feature_dtypes_list in feature_set_to_dtypes.items():
            given_entity_key_dtype = entity_key_to_dtype[entity_key]
            entity_column_tuple = [
                column_tuple
                for column_tuple in feature_dtypes_list
                if column_tuple[0] == entity_key
            ]
            if not entity_column_tuple:
                raise QwakException(
                    f"Did not find entity key [{entity_key}] in the table of [{feature_set_name}] "
                    f"- existing columns are: "
                    f"{[column_tuple[0] for column_tuple in feature_dtypes_list]}"
                )
            actual_entity_type = entity_column_tuple[0][1]
            if actual_entity_type == given_entity_key_dtype:
                continue
            else:
                try:
                    population[entity_key] = population[entity_key].astype(
                        sql_type_to_pandas[actual_entity_type]
                    )
                    print(
                        f"Entity [{entity_key}] given type [{given_entity_key_dtype}] "
                        f"was not aligned with actual type [{actual_entity_type}] - casted to correct type"
                    )
                except ValueError as e:
                    raise QwakException(
                        f"Mismatched entity type for [{entity_key}] - [{given_entity_key_dtype}] "
                        f"- failed to cast to actual type [{actual_entity_type}], Error: {e}"
                    )

        return population

    def _validate_and_get_entity_keys_dtypes(
        self,
        entity_key_to_features: Dict[str, List[str]],
        population_df: "pd.DataFrame",
    ) -> Dict[str, str]:
        """
        Args:
            entity_key_to_features: a dictionary { entity_key(s) -> features list }.
            population_df: a pandas data-frame with a point in time column
                           and a column for each entity key defined at entity_key_to_features.

        Returns:
            dictionary of entity key to it's dtype
        """
        supported_dtypes_to_actual_type = {
            "object": "string",
            "int32": "integer",
            "int64": "integer",
        }
        entity_key_to_dtype = {}
        for entity_key in entity_key_to_features.keys():
            entity_pandas_dtype = population_df.dtypes[entity_key].name
            if entity_pandas_dtype not in supported_dtypes_to_actual_type:
                raise QwakException(
                    f"Got an unsupported dtype for the entity key "
                    f"[{entity_key}] - [{entity_pandas_dtype}]"
                )
            entity_key_to_dtype[entity_key] = supported_dtypes_to_actual_type[
                entity_pandas_dtype
            ]
        return entity_key_to_dtype

    def _run_ranges_query(
        self,
        feature_name_to_feature_list: defaultdict,
        start_date: datetime,
        end_date: datetime,
    ):
        result_dfs = []
        features_set_by_entity = defaultdict(lambda: defaultdict(set))
        for (
            (entity_key, feature_set_name),
            feature_list,
        ) in feature_name_to_feature_list.items():
            for feature in feature_list:
                feature_set_name = feature.split(".")[0]
                features_set_by_entity[entity_key][feature_set_name].add(feature)

        for entity_key, features_dict in features_set_by_entity.items():
            entity_dfs = []
            for feature_set_name, feature_list in features_dict.items():
                offline_feature_store_full_path, table_path = self.get_fs_full_path(
                    feature_set_name
                )

                features = ", ".join(
                    [
                        f"{offline_feature_store_full_path}.{self.quotes}{feature}{self.quotes}"
                        for feature in feature_list
                    ]
                )

                where_part = (
                    "WHERE "
                    f"{table_path}.{self.FS_START_TIME_COLUMN} >= timestamp '{start_date}' "
                    f"AND ({self.FS_END_TIME_COLUMN} <= "
                    f"timestamp '{end_date}' OR {table_path}.{self.FS_END_TIME_COLUMN} IS NULL) "
                    f"AND {table_path}.{self.FS_START_TIME_COLUMN} < timestamp '{end_date}'"
                )

                full_sql = (
                    f"SELECT {offline_feature_store_full_path}.{self.quotes}{entity_key}{self.quotes}, "  # nosec B608
                    f"{offline_feature_store_full_path}.{self.quotes}{self.FS_START_TIME_COLUMN}{self.quotes}, "
                    f"{features} "
                    f"FROM {offline_feature_store_full_path} "
                    f"{where_part}"
                )

                entity_dfs.append(self.query_engine.read_pandas_from_query(full_sql))

            entity_final_df = reduce(
                lambda left, right: pd.merge(
                    left, right, on=[entity_key, self.FS_START_TIME_COLUMN], how="outer"
                ),
                entity_dfs,
            )
            result_dfs.append(entity_final_df.reset_index(drop=True))

        return result_dfs[0] if len(result_dfs) == 1 else result_dfs

    def _run_point_in_time_query(
        self,
        feature_name_to_feature_list: defaultdict,
        uploaded_population_path: str,
        point_in_time_column_name: str,
        population_list: list,
    ) -> "pd.DataFrame":
        """
        creates SQL query for pint in time feature fetching based on population and requested features
        """
        dfs = []

        for index, ((entity_key, feature_set_name), feature_list) in enumerate(
            feature_name_to_feature_list.items()
        ):
            offline_feature_store_full_path, table_path = self.get_fs_full_path(
                feature_set_name
            )

            join_part = self._get_join_population_sql(
                entity_key, offline_feature_store_full_path, uploaded_population_path
            )

            point_in_time_column_full_path = (
                f"{uploaded_population_path}.{point_in_time_column_name}"
            )

            where_part = (
                "WHERE "
                f"{point_in_time_column_full_path} >= "
                f"{table_path}.{self.FS_START_TIME_COLUMN} "
                f"AND ({point_in_time_column_full_path} < "
                f"{table_path}.{self.FS_END_TIME_COLUMN} OR "
                f"{table_path}.{self.FS_END_TIME_COLUMN} IS NULL)"
            )

            features = ", ".join(
                [
                    f"{offline_feature_store_full_path}.{self.quotes}{feature}{self.quotes} as {self.quotes}{feature}{self.quotes}"
                    for feature in feature_list
                ]
            )

            final_query_features = ", ".join(
                [
                    f"filtered_features.{self.quotes}{feature}{self.quotes} as {self.quotes}{feature}{self.quotes}"
                    for feature in feature_list
                ]
            )

            full_sql = (
                f"WITH "  # nosec B608
                "filtered_features AS ( "
                f"SELECT {uploaded_population_path}.*, {features} "
                f"FROM {uploaded_population_path} "
                f"{join_part} "
                f"{where_part} "
                ") "
                f"SELECT population.*, {final_query_features} "
                f"FROM {uploaded_population_path} population "
                f"LEFT JOIN filtered_features "
                f"ON population.{entity_key} = filtered_features.{entity_key} "
                f"AND population.{point_in_time_column_name} = filtered_features.{point_in_time_column_name}"
            )

            dfs.append(
                self.query_engine.read_pandas_from_query(
                    full_sql, [point_in_time_column_name]
                )
            )
        return self._merge_query_dataframes_results(dfs, population_list)

    @staticmethod
    def _merge_query_dataframes_results(
        dfs: List["pd.DataFrame"], population_list
    ) -> "pd.DataFrame":
        """
        merges query result dataframes according to population list cols
        """
        if dfs:
            df_final = reduce(
                lambda left, right: pd.merge(
                    left, right, on=population_list, how="outer"
                ),
                dfs,
            )

            ordered_cols = population_list + (
                df_final.columns.drop(population_list).tolist()
            )

            return df_final[ordered_cols].reset_index(drop=True)
        else:
            return pd.DataFrame()

    def _get_join_population_sql(
        self, entity_key, offline_feature_store_full_path, uploaded_population_path
    ):
        """
        return a join sql query for uploaded population table
        """
        join_part = (
            f"LEFT JOIN {offline_feature_store_full_path} ON "
            f"{offline_feature_store_full_path}.{entity_key} = "
            f"{uploaded_population_path}.{entity_key} "
        )
        return join_part

    def get_fs_full_path(self, feature_set_name: str) -> Tuple[str, str]:
        offline_feature_store_table_name = self._get_offline_feature_store_full_name(
            feature_set_name
        )
        table_path = f"{self.quotes}{offline_feature_store_table_name}{self.quotes}"
        offline_feature_store_full_path = (
            f"{self.quotes}{self.FS_DB_NAME}{self.quotes}."
            f"{self.quotes}{offline_feature_store_table_name}{self.quotes}"
        )
        return offline_feature_store_full_path, table_path

    def _fs_db_names(self) -> List[str]:
        return [
            database_tuple[0]
            for database_tuple in self.query_engine.run_query("SHOW SCHEMAS")
        ]

    def _fs_tables_names(self) -> List[str]:
        return [
            table_tuple[0]
            for table_tuple in self.query_engine.run_query(
                f"SHOW TABLES IN {self.FS_DB_NAME}"
            )
        ]

    def _get_offline_feature_store_full_name(self, feature_set_name: str) -> str:
        return f"{self.FS_TABLE_NAME_PREFIX}_{feature_set_name}".lower().replace(
            "-", "_"
        )



from copy import deepcopy

from _qwak_proto.qwak.builds.build_url_pb2 import BuildVersioningTagsType
from qwak.inner.build_logic.constants.upload_tag import (
    BUILD_CONFIG_TAG,
    QWAK_SDK_VERSION_TAG,
    MODEL_CODE_TAG,
)

from qwak.inner.build_logic.interface.step_inteface import Step


class StartRemoteBuildStep(Step):
    STEP_DESCRIPTION = "Start remote build"

    def description(self) -> str:
        return self.STEP_DESCRIPTION

    def execute(self) -> None:
        self.build_logger.info(f"Start remote build - {self.context.build_id}")
        config_copy = deepcopy(self.config)
        config_copy.build_properties.build_id = self.context.build_id
        self.context.client_builds_orchestrator.build_model(
            build_conf=config_copy,
            build_v1_flag=False,
            build_config_url=self.get_download_url(BUILD_CONFIG_TAG),
            qwak_sdk_version_url=self.get_download_url(QWAK_SDK_VERSION_TAG),
            resolved_model_url=self.get_download_url(MODEL_CODE_TAG),
            git_commit_id=self.context.git_commit_id,
            sdk_version=self.context.qwak_sdk_version,
        )
        self.build_logger.info("Remote build started successfully")

    def get_download_url(self, tag: str) -> str:
        return (
            self.context.client_builds_orchestrator.get_build_versioning_download_url(
                model_id=self.config.build_properties.model_id,
                build_id=self.context.build_id,
                tag=tag,
                tag_type=BuildVersioningTagsType.FILE_TAG_TYPE,
            ).download_url
        )



from qwak.clients.administration.eco_system.client import EcosystemClient


class EcosystemUtils:
    def __init__(self):
        self._client = EcosystemClient()

    def get_default_environment_id(self) -> str:
        user_context = EcosystemClient().get_authenticated_user_context().user
        default_environment_id: str = (
            user_context.account_details.default_environment_id
        )
        return default_environment_id



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/feature_store/features/feature_set_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.feature_store.features import feature_set_state_pb2 as qwak_dot_feature__store_dot_features_dot_feature__set__state__pb2
from _qwak_proto.qwak.feature_store.features import feature_set_pb2 as qwak_dot_feature__store_dot_features_dot_feature__set__pb2
from _qwak_proto.qwak.feature_store.features import feature_set_types_pb2 as qwak_dot_feature__store_dot_features_dot_feature__set__types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5qwak/feature_store/features/feature_set_service.proto\x12\x1bqwak.feature.store.features\x1a\x33qwak/feature_store/features/feature_set_state.proto\x1a-qwak/feature_store/features/feature_set.proto\x1a\x33qwak/feature_store/features/feature_set_types.proto\"b\n\x19RegisterFeatureSetRequest\x12\x45\n\x10\x66\x65\x61ture_set_spec\x18\x01 \x01(\x0b\x32+.qwak.feature.store.features.FeatureSetSpec\"Z\n\x1aRegisterFeatureSetResponse\x12<\n\x0b\x66\x65\x61ture_set\x18\x01 \x01(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"x\n\x17UpdateFeatureSetRequest\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\x12\x45\n\x10\x66\x65\x61ture_set_spec\x18\x02 \x01(\x0b\x32+.qwak.feature.store.features.FeatureSetSpec\"X\n\x18UpdateFeatureSetResponse\x12<\n\x0b\x66\x65\x61ture_set\x18\x01 \x01(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"2\n\x18GetFeatureSetByIdRequest\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\"Y\n\x19GetFeatureSetByIdResponse\x12<\n\x0b\x66\x65\x61ture_set\x18\x01 \x01(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"6\n\x1aGetFeatureSetByNameRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\"[\n\x1bGetFeatureSetByNameResponse\x12<\n\x0b\x66\x65\x61ture_set\x18\x01 \x01(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"1\n\x17\x44\x65leteFeatureSetRequest\x12\x16\n\x0e\x66\x65\x61ture_set_id\x18\x01 \x01(\t\"\x1a\n\x18\x44\x65leteFeatureSetResponse\"\x18\n\x16ListFeatureSetsRequest\"\\\n\x17ListFeatureSetsResponse\x12\x41\n\x10\x66\x65\x61ture_families\x18\x01 \x03(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"5\n\x19RunBatchFeatureSetRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\"\x1c\n\x1aRunBatchFeatureSetResponse\"p\n RunParquetBatchFeatureSetRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\x12\x18\n\x10parquet_location\x18\x02 \x01(\t\x12\x18\n\x10timestamp_column\x18\x03 \x01(\t\"#\n!RunParquetBatchFeatureSetResponse\"2\n\x16\x43reateUploadURLRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\"x\n\x18\x43reateUploadURLRequestV1\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\x12\x43\n\x0etransformation\x18\x02 \x01(\x0b\x32+.qwak.feature.store.features.Transformation\"I\n.CreateRealTimeFeatureExtractorUploadURLRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"-\n\x17\x43reateUploadURLResponse\x12\x12\n\nupload_url\x18\x01 \x01(\t\"\x89\x02\n\x1cUpdateFeatureSetStateRequest\x12m\n\x13\x66\x65\x61ture_names_state\x18\x01 \x03(\x0b\x32P.qwak.feature.store.features.UpdateFeatureSetStateRequest.FeatureNamesStateEntry\x12\x16\n\x0e\x65nvironment_id\x18\x02 \x01(\t\x1a\x62\n\x16\x46\x65\x61tureNamesStateEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x37\n\x05value\x18\x02 \x01(\x0b\x32(.qwak.feature.store.features.StateUpdate:\x02\x38\x01\"e\n\x0bStateUpdate\x12V\n\x13\x66\x65\x61ture_names_state\x18\x01 \x01(\x0e\x32\x39.qwak.feature.store.features.FeatureSetLastExecutionState\"\x1f\n\x1dUpdateFeatureSetStateResponse\"K\n\x1fGetFeatureSetStateByNameRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\x12\x0e\n\x06run_id\x18\x02 \x01(\x03\"x\n GetFeatureSetStateByNameResponse\x12T\n\x11\x66\x65\x61ture_set_state\x18\x01 \x01(\x0e\x32\x39.qwak.feature.store.features.FeatureSetLastExecutionState\"9\n\x1dGetFeatureSetStateByIdRequest\x12\x18\n\x10\x66\x65\x61ture_set_name\x18\x01 \x01(\t\"^\n\x1eGetFeatureSetStateByIdResponse\x12<\n\x0b\x66\x65\x61ture_set\x18\x01 \x01(\x0b\x32\'.qwak.feature.store.features.FeatureSet\"1\n\x16PauseFeaturesetRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"\x19\n\x17PauseFeaturesetResponse\"2\n\x17ResumeFeaturesetRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"\x1a\n\x18ResumeFeaturesetResponse\">\n#GetFeaturesetSchedulingStateRequest\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x01 \x01(\t\"m\n$GetFeaturesetSchedulingStateResponse\x12\x45\n\x05state\x18\x01 \x01(\x0e\x32\x36.qwak.feature.store.features.FeaturesetSchedulingState2\xe4\x12\n\x11\x46\x65\x61tureSetService\x12\x85\x01\n\x12RegisterFeatureSet\x12\x36.qwak.feature.store.features.RegisterFeatureSetRequest\x1a\x37.qwak.feature.store.features.RegisterFeatureSetResponse\x12\x7f\n\x10UpdateFeatureSet\x12\x34.qwak.feature.store.features.UpdateFeatureSetRequest\x1a\x35.qwak.feature.store.features.UpdateFeatureSetResponse\x12\x82\x01\n\x11GetFeatureSetById\x12\x35.qwak.feature.store.features.GetFeatureSetByIdRequest\x1a\x36.qwak.feature.store.features.GetFeatureSetByIdResponse\x12\x88\x01\n\x13GetFeatureSetByName\x12\x37.qwak.feature.store.features.GetFeatureSetByNameRequest\x1a\x38.qwak.feature.store.features.GetFeatureSetByNameResponse\x12\x7f\n\x10\x44\x65leteFeatureSet\x12\x34.qwak.feature.store.features.DeleteFeatureSetRequest\x1a\x35.qwak.feature.store.features.DeleteFeatureSetResponse\x12|\n\x0fListFeatureSets\x12\x33.qwak.feature.store.features.ListFeatureSetsRequest\x1a\x34.qwak.feature.store.features.ListFeatureSetsResponse\x12\x85\x01\n\x12RunBatchFeatureSet\x12\x36.qwak.feature.store.features.RunBatchFeatureSetRequest\x1a\x37.qwak.feature.store.features.RunBatchFeatureSetResponse\x12\x9a\x01\n\x19RunParquetBatchFeatureSet\x12=.qwak.feature.store.features.RunParquetBatchFeatureSetRequest\x1a>.qwak.feature.store.features.RunParquetBatchFeatureSetResponse\x12|\n\x0f\x43reateUploadURL\x12\x33.qwak.feature.store.features.CreateUploadURLRequest\x1a\x34.qwak.feature.store.features.CreateUploadURLResponse\x12\x80\x01\n\x11\x43reateUploadURLV1\x12\x35.qwak.feature.store.features.CreateUploadURLRequestV1\x1a\x34.qwak.feature.store.features.CreateUploadURLResponse\x12\xac\x01\n\'CreateRealTimeFeatureExtractorUploadURL\x12K.qwak.feature.store.features.CreateRealTimeFeatureExtractorUploadURLRequest\x1a\x34.qwak.feature.store.features.CreateUploadURLResponse\x12\x8e\x01\n\x15UpdateFeatureSetState\x12\x39.qwak.feature.store.features.UpdateFeatureSetStateRequest\x1a:.qwak.feature.store.features.UpdateFeatureSetStateResponse\x12\x97\x01\n\x18GetFeatureSetStateByName\x12<.qwak.feature.store.features.GetFeatureSetStateByNameRequest\x1a=.qwak.feature.store.features.GetFeatureSetStateByNameResponse\x12\x91\x01\n\x16GetFeatureSetStateById\x12:.qwak.feature.store.features.GetFeatureSetStateByIdRequest\x1a;.qwak.feature.store.features.GetFeatureSetStateByIdResponse\x12|\n\x0fPauseFeatureset\x12\x33.qwak.feature.store.features.PauseFeaturesetRequest\x1a\x34.qwak.feature.store.features.PauseFeaturesetResponse\x12\x7f\n\x10ResumeFeatureset\x12\x34.qwak.feature.store.features.ResumeFeaturesetRequest\x1a\x35.qwak.feature.store.features.ResumeFeaturesetResponse\x12\xa3\x01\n\x1cGetFeaturesetSchedulingState\x12@.qwak.feature.store.features.GetFeaturesetSchedulingStateRequest\x1a\x41.qwak.feature.store.features.GetFeaturesetSchedulingStateResponseB]\n(com.qwak.ai.feature.store.management.apiP\x01Z/qwak/featurestore/features;featurestorefeaturesb\x06proto3')



_REGISTERFEATURESETREQUEST = DESCRIPTOR.message_types_by_name['RegisterFeatureSetRequest']
_REGISTERFEATURESETRESPONSE = DESCRIPTOR.message_types_by_name['RegisterFeatureSetResponse']
_UPDATEFEATURESETREQUEST = DESCRIPTOR.message_types_by_name['UpdateFeatureSetRequest']
_UPDATEFEATURESETRESPONSE = DESCRIPTOR.message_types_by_name['UpdateFeatureSetResponse']
_GETFEATURESETBYIDREQUEST = DESCRIPTOR.message_types_by_name['GetFeatureSetByIdRequest']
_GETFEATURESETBYIDRESPONSE = DESCRIPTOR.message_types_by_name['GetFeatureSetByIdResponse']
_GETFEATURESETBYNAMEREQUEST = DESCRIPTOR.message_types_by_name['GetFeatureSetByNameRequest']
_GETFEATURESETBYNAMERESPONSE = DESCRIPTOR.message_types_by_name['GetFeatureSetByNameResponse']
_DELETEFEATURESETREQUEST = DESCRIPTOR.message_types_by_name['DeleteFeatureSetRequest']
_DELETEFEATURESETRESPONSE = DESCRIPTOR.message_types_by_name['DeleteFeatureSetResponse']
_LISTFEATURESETSREQUEST = DESCRIPTOR.message_types_by_name['ListFeatureSetsRequest']
_LISTFEATURESETSRESPONSE = DESCRIPTOR.message_types_by_name['ListFeatureSetsResponse']
_RUNBATCHFEATURESETREQUEST = DESCRIPTOR.message_types_by_name['RunBatchFeatureSetRequest']
_RUNBATCHFEATURESETRESPONSE = DESCRIPTOR.message_types_by_name['RunBatchFeatureSetResponse']
_RUNPARQUETBATCHFEATURESETREQUEST = DESCRIPTOR.message_types_by_name['RunParquetBatchFeatureSetRequest']
_RUNPARQUETBATCHFEATURESETRESPONSE = DESCRIPTOR.message_types_by_name['RunParquetBatchFeatureSetResponse']
_CREATEUPLOADURLREQUEST = DESCRIPTOR.message_types_by_name['CreateUploadURLRequest']
_CREATEUPLOADURLREQUESTV1 = DESCRIPTOR.message_types_by_name['CreateUploadURLRequestV1']
_CREATEREALTIMEFEATUREEXTRACTORUPLOADURLREQUEST = DESCRIPTOR.message_types_by_name['CreateRealTimeFeatureExtractorUploadURLRequest']
_CREATEUPLOADURLRESPONSE = DESCRIPTOR.message_types_by_name['CreateUploadURLResponse']
_UPDATEFEATURESETSTATEREQUEST = DESCRIPTOR.message_types_by_name['UpdateFeatureSetStateRequest']
_UPDATEFEATURESETSTATEREQUEST_FEATURENAMESSTATEENTRY = _UPDATEFEATURESETSTATEREQUEST.nested_types_by_name['FeatureNamesStateEntry']
_STATEUPDATE = DESCRIPTOR.message_types_by_name['StateUpdate']
_UPDATEFEATURESETSTATERESPONSE = DESCRIPTOR.message_types_by_name['UpdateFeatureSetStateResponse']
_GETFEATURESETSTATEBYNAMEREQUEST = DESCRIPTOR.message_types_by_name['GetFeatureSetStateByNameRequest']
_GETFEATURESETSTATEBYNAMERESPONSE = DESCRIPTOR.message_types_by_name['GetFeatureSetStateByNameResponse']
_GETFEATURESETSTATEBYIDREQUEST = DESCRIPTOR.message_types_by_name['GetFeatureSetStateByIdRequest']
_GETFEATURESETSTATEBYIDRESPONSE = DESCRIPTOR.message_types_by_name['GetFeatureSetStateByIdResponse']
_PAUSEFEATURESETREQUEST = DESCRIPTOR.message_types_by_name['PauseFeaturesetRequest']
_PAUSEFEATURESETRESPONSE = DESCRIPTOR.message_types_by_name['PauseFeaturesetResponse']
_RESUMEFEATURESETREQUEST = DESCRIPTOR.message_types_by_name['ResumeFeaturesetRequest']
_RESUMEFEATURESETRESPONSE = DESCRIPTOR.message_types_by_name['ResumeFeaturesetResponse']
_GETFEATURESETSCHEDULINGSTATEREQUEST = DESCRIPTOR.message_types_by_name['GetFeaturesetSchedulingStateRequest']
_GETFEATURESETSCHEDULINGSTATERESPONSE = DESCRIPTOR.message_types_by_name['GetFeaturesetSchedulingStateResponse']
RegisterFeatureSetRequest = _reflection.GeneratedProtocolMessageType('RegisterFeatureSetRequest', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERFEATURESETREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.RegisterFeatureSetRequest)
  })
_sym_db.RegisterMessage(RegisterFeatureSetRequest)

RegisterFeatureSetResponse = _reflection.GeneratedProtocolMessageType('RegisterFeatureSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERFEATURESETRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.RegisterFeatureSetResponse)
  })
_sym_db.RegisterMessage(RegisterFeatureSetResponse)

UpdateFeatureSetRequest = _reflection.GeneratedProtocolMessageType('UpdateFeatureSetRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEFEATURESETREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.UpdateFeatureSetRequest)
  })
_sym_db.RegisterMessage(UpdateFeatureSetRequest)

UpdateFeatureSetResponse = _reflection.GeneratedProtocolMessageType('UpdateFeatureSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEFEATURESETRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.UpdateFeatureSetResponse)
  })
_sym_db.RegisterMessage(UpdateFeatureSetResponse)

GetFeatureSetByIdRequest = _reflection.GeneratedProtocolMessageType('GetFeatureSetByIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETBYIDREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.GetFeatureSetByIdRequest)
  })
_sym_db.RegisterMessage(GetFeatureSetByIdRequest)

GetFeatureSetByIdResponse = _reflection.GeneratedProtocolMessageType('GetFeatureSetByIdResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETBYIDRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.GetFeatureSetByIdResponse)
  })
_sym_db.RegisterMessage(GetFeatureSetByIdResponse)

GetFeatureSetByNameRequest = _reflection.GeneratedProtocolMessageType('GetFeatureSetByNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETBYNAMEREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.GetFeatureSetByNameRequest)
  })
_sym_db.RegisterMessage(GetFeatureSetByNameRequest)

GetFeatureSetByNameResponse = _reflection.GeneratedProtocolMessageType('GetFeatureSetByNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETBYNAMERESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.GetFeatureSetByNameResponse)
  })
_sym_db.RegisterMessage(GetFeatureSetByNameResponse)

DeleteFeatureSetRequest = _reflection.GeneratedProtocolMessageType('DeleteFeatureSetRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEFEATURESETREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.DeleteFeatureSetRequest)
  })
_sym_db.RegisterMessage(DeleteFeatureSetRequest)

DeleteFeatureSetResponse = _reflection.GeneratedProtocolMessageType('DeleteFeatureSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEFEATURESETRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.DeleteFeatureSetResponse)
  })
_sym_db.RegisterMessage(DeleteFeatureSetResponse)

ListFeatureSetsRequest = _reflection.GeneratedProtocolMessageType('ListFeatureSetsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTFEATURESETSREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.ListFeatureSetsRequest)
  })
_sym_db.RegisterMessage(ListFeatureSetsRequest)

ListFeatureSetsResponse = _reflection.GeneratedProtocolMessageType('ListFeatureSetsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTFEATURESETSRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.ListFeatureSetsResponse)
  })
_sym_db.RegisterMessage(ListFeatureSetsResponse)

RunBatchFeatureSetRequest = _reflection.GeneratedProtocolMessageType('RunBatchFeatureSetRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNBATCHFEATURESETREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.RunBatchFeatureSetRequest)
  })
_sym_db.RegisterMessage(RunBatchFeatureSetRequest)

RunBatchFeatureSetResponse = _reflection.GeneratedProtocolMessageType('RunBatchFeatureSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNBATCHFEATURESETRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.RunBatchFeatureSetResponse)
  })
_sym_db.RegisterMessage(RunBatchFeatureSetResponse)

RunParquetBatchFeatureSetRequest = _reflection.GeneratedProtocolMessageType('RunParquetBatchFeatureSetRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNPARQUETBATCHFEATURESETREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.RunParquetBatchFeatureSetRequest)
  })
_sym_db.RegisterMessage(RunParquetBatchFeatureSetRequest)

RunParquetBatchFeatureSetResponse = _reflection.GeneratedProtocolMessageType('RunParquetBatchFeatureSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNPARQUETBATCHFEATURESETRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.RunParquetBatchFeatureSetResponse)
  })
_sym_db.RegisterMessage(RunParquetBatchFeatureSetResponse)

CreateUploadURLRequest = _reflection.GeneratedProtocolMessageType('CreateUploadURLRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEUPLOADURLREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.CreateUploadURLRequest)
  })
_sym_db.RegisterMessage(CreateUploadURLRequest)

CreateUploadURLRequestV1 = _reflection.GeneratedProtocolMessageType('CreateUploadURLRequestV1', (_message.Message,), {
  'DESCRIPTOR' : _CREATEUPLOADURLREQUESTV1,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.CreateUploadURLRequestV1)
  })
_sym_db.RegisterMessage(CreateUploadURLRequestV1)

CreateRealTimeFeatureExtractorUploadURLRequest = _reflection.GeneratedProtocolMessageType('CreateRealTimeFeatureExtractorUploadURLRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEREALTIMEFEATUREEXTRACTORUPLOADURLREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.CreateRealTimeFeatureExtractorUploadURLRequest)
  })
_sym_db.RegisterMessage(CreateRealTimeFeatureExtractorUploadURLRequest)

CreateUploadURLResponse = _reflection.GeneratedProtocolMessageType('CreateUploadURLResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEUPLOADURLRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.CreateUploadURLResponse)
  })
_sym_db.RegisterMessage(CreateUploadURLResponse)

UpdateFeatureSetStateRequest = _reflection.GeneratedProtocolMessageType('UpdateFeatureSetStateRequest', (_message.Message,), {

  'FeatureNamesStateEntry' : _reflection.GeneratedProtocolMessageType('FeatureNamesStateEntry', (_message.Message,), {
    'DESCRIPTOR' : _UPDATEFEATURESETSTATEREQUEST_FEATURENAMESSTATEENTRY,
    '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
    # @@protoc_insertion_point(class_scope:qwak.feature.store.features.UpdateFeatureSetStateRequest.FeatureNamesStateEntry)
    })
  ,
  'DESCRIPTOR' : _UPDATEFEATURESETSTATEREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.UpdateFeatureSetStateRequest)
  })
_sym_db.RegisterMessage(UpdateFeatureSetStateRequest)
_sym_db.RegisterMessage(UpdateFeatureSetStateRequest.FeatureNamesStateEntry)

StateUpdate = _reflection.GeneratedProtocolMessageType('StateUpdate', (_message.Message,), {
  'DESCRIPTOR' : _STATEUPDATE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.StateUpdate)
  })
_sym_db.RegisterMessage(StateUpdate)

UpdateFeatureSetStateResponse = _reflection.GeneratedProtocolMessageType('UpdateFeatureSetStateResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEFEATURESETSTATERESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.UpdateFeatureSetStateResponse)
  })
_sym_db.RegisterMessage(UpdateFeatureSetStateResponse)

GetFeatureSetStateByNameRequest = _reflection.GeneratedProtocolMessageType('GetFeatureSetStateByNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETSTATEBYNAMEREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.GetFeatureSetStateByNameRequest)
  })
_sym_db.RegisterMessage(GetFeatureSetStateByNameRequest)

GetFeatureSetStateByNameResponse = _reflection.GeneratedProtocolMessageType('GetFeatureSetStateByNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETSTATEBYNAMERESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.GetFeatureSetStateByNameResponse)
  })
_sym_db.RegisterMessage(GetFeatureSetStateByNameResponse)

GetFeatureSetStateByIdRequest = _reflection.GeneratedProtocolMessageType('GetFeatureSetStateByIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETSTATEBYIDREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.GetFeatureSetStateByIdRequest)
  })
_sym_db.RegisterMessage(GetFeatureSetStateByIdRequest)

GetFeatureSetStateByIdResponse = _reflection.GeneratedProtocolMessageType('GetFeatureSetStateByIdResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETSTATEBYIDRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.GetFeatureSetStateByIdResponse)
  })
_sym_db.RegisterMessage(GetFeatureSetStateByIdResponse)

PauseFeaturesetRequest = _reflection.GeneratedProtocolMessageType('PauseFeaturesetRequest', (_message.Message,), {
  'DESCRIPTOR' : _PAUSEFEATURESETREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.PauseFeaturesetRequest)
  })
_sym_db.RegisterMessage(PauseFeaturesetRequest)

PauseFeaturesetResponse = _reflection.GeneratedProtocolMessageType('PauseFeaturesetResponse', (_message.Message,), {
  'DESCRIPTOR' : _PAUSEFEATURESETRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.PauseFeaturesetResponse)
  })
_sym_db.RegisterMessage(PauseFeaturesetResponse)

ResumeFeaturesetRequest = _reflection.GeneratedProtocolMessageType('ResumeFeaturesetRequest', (_message.Message,), {
  'DESCRIPTOR' : _RESUMEFEATURESETREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.ResumeFeaturesetRequest)
  })
_sym_db.RegisterMessage(ResumeFeaturesetRequest)

ResumeFeaturesetResponse = _reflection.GeneratedProtocolMessageType('ResumeFeaturesetResponse', (_message.Message,), {
  'DESCRIPTOR' : _RESUMEFEATURESETRESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.ResumeFeaturesetResponse)
  })
_sym_db.RegisterMessage(ResumeFeaturesetResponse)

GetFeaturesetSchedulingStateRequest = _reflection.GeneratedProtocolMessageType('GetFeaturesetSchedulingStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETSCHEDULINGSTATEREQUEST,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.GetFeaturesetSchedulingStateRequest)
  })
_sym_db.RegisterMessage(GetFeaturesetSchedulingStateRequest)

GetFeaturesetSchedulingStateResponse = _reflection.GeneratedProtocolMessageType('GetFeaturesetSchedulingStateResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESETSCHEDULINGSTATERESPONSE,
  '__module__' : 'qwak.feature_store.features.feature_set_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.features.GetFeaturesetSchedulingStateResponse)
  })
_sym_db.RegisterMessage(GetFeaturesetSchedulingStateResponse)

_FEATURESETSERVICE = DESCRIPTOR.services_by_name['FeatureSetService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n(com.qwak.ai.feature.store.management.apiP\001Z/qwak/featurestore/features;featurestorefeatures'
  _UPDATEFEATURESETSTATEREQUEST_FEATURENAMESSTATEENTRY._options = None
  _UPDATEFEATURESETSTATEREQUEST_FEATURENAMESSTATEENTRY._serialized_options = b'8\001'
  _REGISTERFEATURESETREQUEST._serialized_start=239
  _REGISTERFEATURESETREQUEST._serialized_end=337
  _REGISTERFEATURESETRESPONSE._serialized_start=339
  _REGISTERFEATURESETRESPONSE._serialized_end=429
  _UPDATEFEATURESETREQUEST._serialized_start=431
  _UPDATEFEATURESETREQUEST._serialized_end=551
  _UPDATEFEATURESETRESPONSE._serialized_start=553
  _UPDATEFEATURESETRESPONSE._serialized_end=641
  _GETFEATURESETBYIDREQUEST._serialized_start=643
  _GETFEATURESETBYIDREQUEST._serialized_end=693
  _GETFEATURESETBYIDRESPONSE._serialized_start=695
  _GETFEATURESETBYIDRESPONSE._serialized_end=784
  _GETFEATURESETBYNAMEREQUEST._serialized_start=786
  _GETFEATURESETBYNAMEREQUEST._serialized_end=840
  _GETFEATURESETBYNAMERESPONSE._serialized_start=842
  _GETFEATURESETBYNAMERESPONSE._serialized_end=933
  _DELETEFEATURESETREQUEST._serialized_start=935
  _DELETEFEATURESETREQUEST._serialized_end=984
  _DELETEFEATURESETRESPONSE._serialized_start=986
  _DELETEFEATURESETRESPONSE._serialized_end=1012
  _LISTFEATURESETSREQUEST._serialized_start=1014
  _LISTFEATURESETSREQUEST._serialized_end=1038
  _LISTFEATURESETSRESPONSE._serialized_start=1040
  _LISTFEATURESETSRESPONSE._serialized_end=1132
  _RUNBATCHFEATURESETREQUEST._serialized_start=1134
  _RUNBATCHFEATURESETREQUEST._serialized_end=1187
  _RUNBATCHFEATURESETRESPONSE._serialized_start=1189
  _RUNBATCHFEATURESETRESPONSE._serialized_end=1217
  _RUNPARQUETBATCHFEATURESETREQUEST._serialized_start=1219
  _RUNPARQUETBATCHFEATURESETREQUEST._serialized_end=1331
  _RUNPARQUETBATCHFEATURESETRESPONSE._serialized_start=1333
  _RUNPARQUETBATCHFEATURESETRESPONSE._serialized_end=1368
  _CREATEUPLOADURLREQUEST._serialized_start=1370
  _CREATEUPLOADURLREQUEST._serialized_end=1420
  _CREATEUPLOADURLREQUESTV1._serialized_start=1422
  _CREATEUPLOADURLREQUESTV1._serialized_end=1542
  _CREATEREALTIMEFEATUREEXTRACTORUPLOADURLREQUEST._serialized_start=1544
  _CREATEREALTIMEFEATUREEXTRACTORUPLOADURLREQUEST._serialized_end=1617
  _CREATEUPLOADURLRESPONSE._serialized_start=1619
  _CREATEUPLOADURLRESPONSE._serialized_end=1664
  _UPDATEFEATURESETSTATEREQUEST._serialized_start=1667
  _UPDATEFEATURESETSTATEREQUEST._serialized_end=1932
  _UPDATEFEATURESETSTATEREQUEST_FEATURENAMESSTATEENTRY._serialized_start=1834
  _UPDATEFEATURESETSTATEREQUEST_FEATURENAMESSTATEENTRY._serialized_end=1932
  _STATEUPDATE._serialized_start=1934
  _STATEUPDATE._serialized_end=2035
  _UPDATEFEATURESETSTATERESPONSE._serialized_start=2037
  _UPDATEFEATURESETSTATERESPONSE._serialized_end=2068
  _GETFEATURESETSTATEBYNAMEREQUEST._serialized_start=2070
  _GETFEATURESETSTATEBYNAMEREQUEST._serialized_end=2145
  _GETFEATURESETSTATEBYNAMERESPONSE._serialized_start=2147
  _GETFEATURESETSTATEBYNAMERESPONSE._serialized_end=2267
  _GETFEATURESETSTATEBYIDREQUEST._serialized_start=2269
  _GETFEATURESETSTATEBYIDREQUEST._serialized_end=2326
  _GETFEATURESETSTATEBYIDRESPONSE._serialized_start=2328
  _GETFEATURESETSTATEBYIDRESPONSE._serialized_end=2422
  _PAUSEFEATURESETREQUEST._serialized_start=2424
  _PAUSEFEATURESETREQUEST._serialized_end=2473
  _PAUSEFEATURESETRESPONSE._serialized_start=2475
  _PAUSEFEATURESETRESPONSE._serialized_end=2500
  _RESUMEFEATURESETREQUEST._serialized_start=2502
  _RESUMEFEATURESETREQUEST._serialized_end=2552
  _RESUMEFEATURESETRESPONSE._serialized_start=2554
  _RESUMEFEATURESETRESPONSE._serialized_end=2580
  _GETFEATURESETSCHEDULINGSTATEREQUEST._serialized_start=2582
  _GETFEATURESETSCHEDULINGSTATEREQUEST._serialized_end=2644
  _GETFEATURESETSCHEDULINGSTATERESPONSE._serialized_start=2646
  _GETFEATURESETSCHEDULINGSTATERESPONSE._serialized_end=2755
  _FEATURESETSERVICE._serialized_start=2758
  _FEATURESETSERVICE._serialized_end=5162
# @@protoc_insertion_point(module_scope)
